
/*
 * File Tracing.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include "Debug/Log.hpp"
#include "Debug/Tracer.hpp"

#include "Tracing.hpp"

namespace Api
{

unsigned Tracing::s_traceStackDepth = 0;

void Tracing::enableTrace(std::string traceName, unsigned depth)
{
  CALL("Tracing::enableTrace");
  ENABLE_TAG_LIMITED(traceName.c_str(), depth);
}

void Tracing::processTraceString(std::string str)
{
  CALL("Tracing::processTraceString");
  PROCESS_TRACE_SPEC_STRING(str);
}

void Tracing::pushTracingState()
{
  CALL("Tracing::pushTracingState");

  PUSH_TAG_STATES();
  s_traceStackDepth++;
}

void Tracing::popTracingState()
{
  CALL("Tracing::popTracingState");

  if(s_traceStackDepth==0) {
    throw "No pushed tracing state left to be popped";
  }
  s_traceStackDepth--;
  POP_TAG_STATES();
}

void Tracing::displayHelp()
{
  CALL("Tracing::displayHelp");
  DISPLAY_HELP();
}

}
