
/*
 * File Tracer.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#include <climits>
#include <iostream>

#include "Tracer.hpp"

#if VDEBUG


#define CP_FIRST_POINT UINT_MAX
#define CP_LAST_POINT 0u








using namespace std;
using namespace Debug;

const char* Tracer::_lastControlPoint;
Tracer* Tracer::_current = 0;
unsigned Tracer::_depth = 0;
unsigned Tracer::_passedControlPoints = 0L;
ControlPointKind Tracer::_lastPointKind = CP_MID;
bool Tracer::_forced = false;

bool Tracer::canWatch = false;


#define WATCH_ADDR 0
#if WATCH_ADDR
void* watchAddrLastValue = (void*)0;
#endif

Tracer::Tracer (const char* fun) 
  :
  _fun (fun),
  _previous (_current)
{

  _current = this;
  controlPoint(_fun,CP_ENTRY);
  _depth++;
} 


Tracer::~Tracer () 
{

  _current = _previous;
  _depth--;
  controlPoint(_fun,CP_EXIT);
} 


void Tracer::controlPoint (const char* description, ControlPointKind cpk)
{








  _lastPointKind = cpk;

#if WATCH_ADDR
  if (canWatch) {
    void* newVal = *((void**)WATCH_ADDR);
    if (newVal != watchAddrLastValue) {
      cout << "W! " << newVal << " (timestamp: "
	   << _passedControlPoints << ")\n";
      watchAddrLastValue = newVal;
      cout << "Stack on " << (cpk == CP_EXIT ? "exiting" : "entering")
	   << " the last function on the stack:\n";
      printStack(cout);
    }
  }
#endif

  _passedControlPoints++;

  _lastControlPoint = description;
  if (_forced ||
      (_passedControlPoints <= CP_LAST_POINT &&
       _passedControlPoints >= CP_FIRST_POINT)) {
    outputLastControlPoint(cout);
  }
} 


void Tracer::controlPoint (const char* description)
{
  controlPoint (description,CP_MID);
} 


void Tracer::outputLastControlPoint (ostream& str)
{
  spaces(str,_depth);
  str << (_lastPointKind == CP_ENTRY ? '+' : 
          _lastPointKind == CP_EXIT ? '-' : ' ')
      << _lastControlPoint 
      << " ("
      << _passedControlPoints 
      << ")\n";
} 


void Tracer::printOnlyStack (ostream& str)
{
  int depth = 0;
  _current->printStack (str, depth);
} 


void Tracer::printStack (ostream& str)
{
  int depth = 0;

  str << "Control points passed: " << _passedControlPoints << "\n"
      << "last control point:\n";
  outputLastControlPoint(str);
  _current->printStack (str, depth);
} 


void Tracer::printStack (ostream& str, int& depth)
{
  if (! this) { 
    return;
  }
  _previous->printStack(str,depth);
  spaces(str,depth);
  str << _fun << "\n";
  depth ++;
} 


void Tracer::spaces (ostream& str,int n)
{
  while (n > 0) {
    n--;
    str << ' ';
  }
} 

#endif 
