
/*
 * File Tracer.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __Tracer__
#  define __Tracer__

#if VDEBUG

#include <iostream>

using namespace std;

namespace Debug {

enum ControlPointKind {
 
  CP_ENTRY,
 
  CP_EXIT,
 
  CP_MID
}; 


class Tracer {
 public:
  explicit Tracer (const char* fun);
  virtual ~Tracer ();
  static void printStack (ostream&);
  static void printOnlyStack (ostream&);
  static void controlPoint (const char* name);
  static unsigned passedControlPoints () { return _passedControlPoints; }
 
  static void forceOutput() { _forced = true; }
 
  static void stopOutput() { _forced = false; }
  static bool canWatch;

 private:
  const char* _fun;
  Tracer* _previous;

  void printStack (ostream&, int& depth);
  static void spaces(ostream& str,int number);

 
  static Tracer* _current;
 
  static unsigned _depth;
 
  static const char* _lastControlPoint;
 
  static unsigned _passedControlPoints;
 
  static ControlPointKind _lastPointKind;
 
  static bool _forced;
  static void controlPoint (const char*, ControlPointKind);
  static void outputLastControlPoint (ostream& str);
};

} 

#  define AUX_CALL_(SEED,Fun) Debug::Tracer _tmp_##SEED##_(Fun);
#  define AUX_CALL(SEED,Fun) AUX_CALL_(SEED,Fun)
#  define CALL(Fun) AUX_CALL(__LINE__,Fun)
#  define CONTROL(description) Debug::Tracer::controlPoint(description)
#  define AFTER(number,command) \
            if (Debug::Tracer::passedControlPoints() >= number) { command };
#  define BETWEEN(number1,number2,command) \
            if (Debug::Tracer::passedControlPoints() >= number1 &&	\
                Debug::Tracer::passedControlPoints() <= number2)	\
              { command };

#else 
#  define CALL(Fun) 
#  define CONTROL(description)
#endif

#ifndef CALL
#error BLIN
#endif

#endif 
