
/*
 * File LiteralMiniIndex.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include <algorithm>
#include "Lib/Allocator.hpp"

#include "LiteralMiniIndex.hpp"

namespace Indexing
{

using namespace Lib;
using namespace Kernel;



bool LiteralMiniIndex::literalHeaderComparator(const Entry& e1, const Entry& e2)
{
  return e1._header<e2._header || ( e1._header==e2._header && e1._weight<e2._weight );
}

LiteralMiniIndex::LiteralMiniIndex(Clause* cl)
: _cnt(cl->length()), _entries(cl->length()+1)
{
  init(cl->literals());
}

LiteralMiniIndex::LiteralMiniIndex(Literal* const * lits, unsigned length)
: _cnt(length), _entries(length+1)
{
  init(lits);
}

void LiteralMiniIndex::init(Literal* const * lits)
{
  ASS_G(_cnt, 0);
  for(unsigned i=0;i<_cnt;i++) {
    _entries[i].init(lits[i]);
  }
  _entries[_cnt].initTerminal();
  std::sort(_entries.begin(), _entries.end()-1,literalHeaderComparator);
}

}
