
/*
 * File ResultSubstitution.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __ResultSubstitution__
#define __ResultSubstitution__

#include "Forwards.hpp"

#include "Lib/SmartPtr.hpp"
#include "Kernel/Term.hpp"


namespace Indexing {

using namespace Kernel;

class ResultSubstitution
{
public:
  virtual ~ResultSubstitution() {}
  virtual TermList applyToQuery(TermList t) { NOT_IMPLEMENTED; }
  virtual Literal* applyToQuery(Literal* l) { NOT_IMPLEMENTED; }
  virtual TermList applyToResult(TermList t) { NOT_IMPLEMENTED; }
  virtual Literal* applyToResult(Literal* l) { NOT_IMPLEMENTED; }

 
  virtual size_t getQueryApplicationWeight(TermList t) { return 0; }
 
  virtual size_t getQueryApplicationWeight(Literal* l) { return 0; }
 
  virtual size_t getResultApplicationWeight(TermList t) { return 0; }
 
  virtual size_t getResultApplicationWeight(Literal* l) { return 0; }

  template<typename T>
  T apply(T t, bool result)
  {
    if(result) {
      return applyToResult(t);
    } else {
      return applyToQuery(t);
    }
  }

 
  template<typename T>
  size_t getApplicationWeight(T t, bool result)
  {
    if(result) {
      return getResultApplicationWeight(t);
    } else {
      return getQueryApplicationWeight(t);
    }
  }

 
  virtual TermList applyToBoundResult(TermList t)
  { return applyToResult(t); }

 
  virtual Literal* applyToBoundResult(Literal* lit)
  { return applyToResult(lit); }

 
  virtual bool isIdentityOnQueryWhenResultBound() {return false;}


 
  virtual TermList applyToBoundQuery(TermList t)
  { return applyToQuery(t); }

 
  virtual bool isIdentityOnResultWhenQueryBound() {return false;}

  virtual RobSubstitution* tryGetRobSubstitution() { return 0; }

  static ResultSubstitutionSP fromSubstitution(RobSubstitution* s,
	  int queryBank, int resultBank);


};


class IdentitySubstitution
: public ResultSubstitution
{
public:
  static ResultSubstitutionSP instance();

  TermList applyToQuery(TermList t) { return t; }
  Literal* applyToQuery(Literal* l) { return l; }
  TermList applyToResult(TermList t) { return t; }
  Literal* applyToResult(Literal* l) { return l; }
  bool isIdentityOnQueryWhenResultBound() {return true;}
};

class DisjunctQueryAndResultVariablesSubstitution
: public ResultSubstitution
{
public:
  TermList applyToQuery(TermList t);
  Literal* applyToQuery(Literal* l);
  TermList applyToResult(TermList t);
  Literal* applyToResult(Literal* l);

 
  bool isIdentityOnQueryWhenResultBound() {return true;}
private:
  struct Applicator;
};

};

#endif
