
/*
 * File TermSharing.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __TermSharing__
#define __TermSharing__

#include "Lib/Set.hpp"
#include "Kernel/Term.hpp"

using namespace Lib;
using namespace Kernel;

namespace Indexing {

class TermSharing
{
public:
  TermSharing();
  ~TermSharing();

  Term* insert(Term*);
  Term* insertRecurrently(Term*);

  Literal* insert(Literal*);
  Literal* insertVariableEquality(Literal* lit,unsigned sort);

  Literal* tryGetOpposite(Literal* l);

 
  inline static unsigned hash(const Literal* l)
  { return l->hash(); }
 
  inline static unsigned hash(const Term* t)
  { return t->hash(); }
  static bool equals(const Term* t1,const Term* t2);

  static bool equals(const Literal* l1, const Literal* l2, bool opposite=false);

  struct OpLitWrapper {
    OpLitWrapper(Literal* l) : l(l) {}
    Literal* l;
  };
  inline static unsigned hash(const OpLitWrapper& w)
  { return w.l->oppositeHash(); }
  static bool equals(const Literal* l1,const OpLitWrapper& w) {
    return equals(l1, w.l, true);
  }

private:
  bool argNormGt(TermList t1, TermList t2);

 
  Set<Term*,TermSharing> _terms;
 
  Set<Literal*,TermSharing> _literals;
 
  unsigned _totalTerms;
 
  unsigned _groundTerms;
 
  unsigned _totalLiterals;
 
  unsigned _groundLiterals;
 
  unsigned _literalInsertions;
 
  unsigned _termInsertions;
}; 

} 

#endif
