
/*
 * File CTFwSubsAndRes.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#include "Kernel/Inference.hpp"

#include "Indexing/Index.hpp"
#include "Indexing/IndexManager.hpp"

#include "Saturation/SaturationAlgorithm.hpp"

#include "Lib/Environment.hpp"
#include "Shell/Statistics.hpp"


#include "CTFwSubsAndRes.hpp"

namespace Inferences
{
using namespace Lib;
using namespace Kernel;
using namespace Indexing;
using namespace Saturation;

void CTFwSubsAndRes::attach(SaturationAlgorithm* salg)
{
  CALL("CTFwSubsAndRes::attach");

  ForwardSimplificationEngine::attach(salg);
  _index=static_cast<ClauseSubsumptionIndex*>(
	  _salg->getIndexManager()->request(FW_SUBSUMPTION_CODE_TREE) );
}

void CTFwSubsAndRes::detach()
{
  CALL("CTFwSubsAndRes::detach");

  _index=0;
  _salg->getIndexManager()->release(FW_SUBSUMPTION_CODE_TREE);
  ForwardSimplificationEngine::detach();
}

Clause* CTFwSubsAndRes::buildSResClause(Clause* cl, unsigned resolvedIndex, Clause* premise)
{
  CALL("CTFwSubsAndRes::buildSResClause");
  
  unsigned clen = cl->length();
  ASS_L(resolvedIndex,clen);

  unsigned newLength = clen-1;

  Inference* inf = new Inference2(Inference::SUBSUMPTION_RESOLUTION, cl, premise);
  Unit::InputType inpType = (Unit::InputType)
  	max(cl->inputType(), premise->inputType());

  Clause* res = new(newLength) Clause(newLength, inpType, inf);

  unsigned next = 0;
  for(unsigned i=0;i<clen;i++) {
    if(i!=resolvedIndex) {
	(*res)[next++] = (*cl)[i];
    }
  }
  ASS_EQ(next,newLength);

  res->setAge(cl->age());

  return res;
}

void CTFwSubsAndRes::perform(Clause* cl, ForwardSimplificationPerformer* simplPerformer)
{
  CALL("CTFwSubsAndRes::perform");
  
  if(cl->length()==0) {
    return;
  }

  TimeCounter tc_fs(TC_FORWARD_SUBSUMPTION);

  Clause::requestAux();

  ClauseSResResultIterator sresIt=_index->getSubsumingOrSResolvingClauses(cl, 
	  _subsumptionResolution);
  while(sresIt.hasNext()) {
    ClauseSResQueryResult res=sresIt.next();
    Clause* premise=res.clause;
    if(premise->hasAux()) {
      
      continue;
    }
    premise->setAux(0);
    if(!simplPerformer->willPerform(premise)) {
      continue;
    }
    
    if(res.resolved) {
      Clause* replacement=buildSResClause(cl, res.resolvedQueryLiteralIndex, premise);
      simplPerformer->perform(premise, replacement);
      env.statistics->forwardSubsumptionResolution++;
    }
    else {
      simplPerformer->perform(premise, 0);
      env.statistics->forwardSubsumed++;
    }
    
    if(!simplPerformer->clauseKept()) {
      goto fin;
    }
  }

fin:
  Clause::releaseAux();
}


}
