
/*
 * File Factoring.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include <utility>

#include "Lib/Comparison.hpp"
#include "Lib/Environment.hpp"
#include "Lib/Int.hpp"
#include "Lib/Metaiterators.hpp"
#include "Lib/PairUtils.hpp"
#include "Lib/VirtualIterator.hpp"

#include "Kernel/Clause.hpp"
#include "Kernel/Unit.hpp"
#include "Kernel/Inference.hpp"
#include "Kernel/LiteralSelector.hpp"
#include "Kernel/RobSubstitution.hpp"

#include "Saturation/SaturationAlgorithm.hpp"

#include "Shell/Statistics.hpp"

#include "Factoring.hpp"

namespace Inferences
{

using namespace Lib;
using namespace Kernel;
using namespace Indexing;
using namespace Saturation;


class Factoring::UnificationsFn
{
public:
  DECL_RETURN_TYPE(VirtualIterator<pair<Literal*,RobSubstitution*> >);
  UnificationsFn(Clause* cl)
  : _cl(cl)
  {
    _subst=RobSubstitutionSP(new RobSubstitution());
  }
  OWN_RETURN_TYPE operator() (pair<unsigned,unsigned> nums)
  {
    CALL("Factoring::UnificationsFn::operator()");

    Literal* l1 = (*_cl)[nums.first];
    Literal* l2 = (*_cl)[nums.second];

    
    ASS(l1!=l2);
    ASS_EQ(_subst->size(),0);

    if(l1->isEquality()) {
      
      return OWN_RETURN_TYPE::getEmpty();
    }

    SubstIterator unifs=_subst->unifiers(l1,0,l2,0, false);
    if(!unifs.hasNext()) {
      return OWN_RETURN_TYPE::getEmpty();
    }
    return pvi( pushPairIntoRightIterator(l2, unifs) );
  }
private:
  Clause* _cl;
  RobSubstitutionSP _subst;
};

class Factoring::ResultsFn
{
public:
  DECL_RETURN_TYPE(Clause*);
  ResultsFn(Clause* cl)
  : _cl(cl), _cLen(cl->length()) {}
  OWN_RETURN_TYPE operator() (pair<Literal*,RobSubstitution*> arg)
  {
    CALL("Factoring::ResultsFn::operator()");

    unsigned newLength = _cLen-1;
    Inference* inf = new Inference1(Inference::FACTORING, _cl);
    Clause* res = new(newLength) Clause(newLength, _cl->inputType(), inf);

    unsigned next = 0;
    Literal* skipped=arg.first;
    for(unsigned i=0;i<_cLen;i++) {
      Literal* curr=(*_cl)[i];
      if(curr!=skipped) {
	(*res)[next++] = arg.second->apply(curr, 0);
      }
    }
    ASS_EQ(next,newLength);

    res->setAge(_cl->age()+1);
    env.statistics->factoring++;

    return res;
  }
private:
  Clause* _cl;
  
  unsigned _cLen;
};

ClauseIterator Factoring::generateClauses(Clause* premise)
{
  CALL("Factoring::generateClauses");

  if(premise->length()<=1) {
    return ClauseIterator::getEmpty();
  }
  if(premise->selected()==1 && _salg->getLiteralSelector().isNegativeForSelection((*premise)[0])) {
    return ClauseIterator::getEmpty();
  }
  return pvi( getMappingIterator(
	  getMapAndFlattenIterator(
		  getCombinationIterator(0u,premise->selected(),premise->length()),
		  UnificationsFn(premise)),
	  ResultsFn(premise)) );
}

}
