
/*
 * File PropositionalToBDDISE.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include "Lib/Environment.hpp"
#include "Lib/Stack.hpp"

#include "Kernel/BDD.hpp"
#include "Kernel/Clause.hpp"
#include "Kernel/InferenceStore.hpp"
#include "Kernel/MainLoop.hpp"
#include "Kernel/Problem.hpp"
#include "Kernel/Signature.hpp"
#include "Kernel/Term.hpp"

#include "Shell/Statistics.hpp"
#include "Shell/Options.hpp"

#include "PropositionalToBDDISE.hpp"

namespace Inferences
{

using namespace Lib;
using namespace Kernel;


Clause* PropositionalToBDDISE::simplify(Clause* c)
{
  CALL("PropositionalToBDDISE::simplify");

  BDD* bdd=BDD::instance();

  ASS(bdd->isFalse(c->prop()));

  unsigned propCnt=0;
  BDDNode* propPart=bdd->getFalse();

  static Stack<Clause*> premises(8);
  premises.reset();

  unsigned clen=c->length();
  for(unsigned i=0;i<clen;i++) {
    Literal* lit=(*c)[i];

    if(canBddize(lit)) {
      propCnt++;
    }
  }

  if(!propCnt) {
    return c;
  }



  unsigned nlen=clen-propCnt;

  Inference* inf=new Inference1(Inference::BDDZATION, c);

  Clause* newCl=new(nlen) Clause(nlen, c->inputType(), inf);
  newCl->setAge(c->age());

  List<int>* propVars=0;

  unsigned newIndex=0;
  for(unsigned i=0;i<clen;i++) {
    Literal* lit=(*c)[i];
    if(canBddize(lit)) {
      int name = getPropPredName(lit);
      List<int>::push(name, propVars);
      propPart = bdd->disjunction(propPart, bdd->getAtomic(name, lit->isPositive()));
    } else {
      (*newCl)[newIndex++]=lit;
    }
  }
  ASS_EQ(newIndex, nlen);

  newCl->initProp(propPart);

  if(bdd->isTrue(propPart)) {
    newCl->destroyIfUnnecessary();
    return 0;
  }

  InferenceStore::instance()->recordNonPropInference(newCl);
  InferenceStore::instance()->recordBddizeVars(newCl, propVars);

  return newCl;
}

bool PropositionalToBDDISE::colorOkForBDD(Color c)
{
  return c==COLOR_TRANSPARENT;
}

bool PropositionalToBDDISE::canBddize(Literal* l)
{
  return l->arity()==0 && colorOkForBDD(l->color()) &&
    (!env.options->showSymbolElimination() || l->skip()) &&
    !env.signature->getPredicate(l->functor())->cfName();
}
int PropositionalToBDDISE::getPropPredName(Literal* lit)
{
  CALL("PropositionalToBDDISE::getPropPredName");
  ASS_EQ(lit->arity(), 0);

  unsigned pred=lit->functor();
  int* pname;
  if(_propPredNames.getValuePtr(pred, pname)) {
    *pname=BDD::instance()->getNewVar(pred);
    if(_parent) {
      _parent->getProblem().addBDDVarMeaning(*pname, Problem::BDDMeaningSpec(Literal::positiveLiteral(lit), 0));
    }
  }
  return *pname;
}

}
