
/*
 * File BDDClausifier.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __BDDClausifier__
#define __BDDClausifier__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"

namespace Kernel {

using namespace Lib;
using namespace SAT;

class BDDClausifier {
public:
  BDDClausifier(bool subsumptionResolution, bool naming=false);

  void clausify(BDDNode* node, SATClauseStack& acc);

  unsigned getCNFVarCount();
private:
  struct CNFStackRec;

  void clausify(BDDNode* node, SATClauseStack& acc, unsigned givenName);
  void clausifyWithSR(BDDNode* node, SATClauseStack& acc, unsigned givenName);
  void clausifyWithoutSR(BDDNode* node, SATClauseStack& acc, unsigned givenName);

  SATClause* buildClause(unsigned givenName, Stack<CNFStackRec>& stack,
      unsigned resolvedCnt=0, unsigned nodeName=0);

  unsigned getName(BDDNode* node);

  bool shouldName(BDDNode* node);

  unsigned assignName(BDDNode* node, SATClauseStack& acc);
  unsigned getCNFVar(unsigned bddVar);


  void introduceNames(BDDNode* node, SATClauseStack& acc);


  unsigned _nextCNFVar;
  unsigned _maxBDDVar;
  bool _useSR;
  bool _naming;
  DHMap<BDDNode*, unsigned> _names;
  DHMap<unsigned, unsigned> _bdd2cnfVars;
};

}

#endif 
