
/*
 * File BDDConjunction.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include "Lib/Environment.hpp"
#include "Lib/TimeCounter.hpp"

#include "SAT/TWLSolver.hpp"

#include "Shell/Options.hpp"

#include "BDD.hpp"

#include "BDDConjunction.hpp"

namespace Kernel
{

BDDConjunction::BDDConjunction(const Options& opt)
: _isFalse(false),
  _clausifier(opt.satSolverWithSubsumptionResolution(), opt.satSolverWithNaming()),
  _solver(new TWLSolver(opt))
{
}


void BDDConjunction::addNode(BDDNode* n)
{
  CALL("BDDConjunction::addNode");

  if(_isFalse) {
    return;
  }

  BDD* bdd=BDD::instance();

  if(bdd->isConstant(n)) {
    if(bdd->isFalse(n)) {
      _isFalse=true;
    } else {
      ASS(bdd->isTrue(n));
    }
    return;
  }



  static SATClauseStack acc;
  acc.reset();

  {
    TimeCounter tc(TC_BDD_CLAUSIFICATION);
    _clausifier.clausify(n, acc);
  }

  TimeCounter tc(TC_SAT_SOLVER);

  _solver->ensureVarCnt(_clausifier.getCNFVarCount());
  _solver->addClauses(pvi( SATClauseStack::Iterator(acc) ));

  if(_solver->getStatus()==TWLSolver::UNSATISFIABLE) {
    _isFalse=true;
  }

  return;
}

}
