
/*
 * File ColorHelper.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __ColorHelper__
#define __ColorHelper__

#include "Forwards.hpp"

#include "Debug/Assertion.hpp"

#include "Lib/DHMap.hpp"
#include "Lib/Environment.hpp"

namespace Kernel {

using namespace Lib;
using namespace Saturation;

class ColorHelper {
public:

 
  static Color combine(Color c1, Color c2) {
    CALL("ColorHelper::combine");
    ASS(env.colorUsed || (c1|c2)!=COLOR_INVALID);
    return static_cast<Color>(c1|c2);
  }

 
  static bool compatible(Color c1, Color c2) {
    CALL("ColorHelper::compatible");
    return combine(c1,c2)!=COLOR_INVALID;
  }

  static bool isTransparent(bool predicate, unsigned functor);
  static bool hasColoredPredicates(Clause* c);
  static bool hasOnlyColoredConstants(Clause* c);
  static Clause* skolemizeColoredConstants(Clause* c);
  static Clause* skolemizeColoredTerms(Clause* c);

  static void tryUnblock(Clause* c, SaturationAlgorithm* salg);

private:
  typedef DHMap<Term*, Term*> TermMap;

  static void ensureSkolemReplacement(Term* t, TermMap& map);
  static void collectSkolemReplacements(Clause* c, TermMap& map);

  static Term* applyReplacement(Term* t, TermMap& map);

  static void collectColoredConstants(Clause* c, Stack<Term*>& acc);
};

}

#endif 
