
/*
 * File Curryfier.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __Curryfier__
#define __Curryfier__

#include "Lib/Environment.hpp"
#include "Lib/DHMap.hpp"
#include "Lib/DArray.hpp"
#include "Lib/ArrayMap.hpp"

#include "Indexing/TermSharing.hpp"

#include "Term.hpp"
#include "Signature.hpp"

namespace Kernel {

using namespace Lib;

class Curryfier
{
public:

  TermList curryfy(TermList t)
  {
    if(t.isVar() || (t.isTerm() && t.term()->arity()==0) ) {
      return t;
    }
    Term* trm=t.term();
    if(_termCache.find(trm)) {
      return _termCache.get(trm);
    }







    TermList res=getCurryFn(trm->functor());
    TermList *arg=trm->args();
    while(!arg->isEmpty()) {
      res=getCurryFn(curryfy(*arg),res);
      arg=arg->next();
    }









    return res;
  }

  static Curryfier* instance()
  {
    static Curryfier* res=0;
    if(!res) {
      res=new Curryfier();
    }
    return res;
  }
private:
  TermList getCurryFn(TermList fn, TermList arg)
  {
    Term* t = new(2) Term;
    t->makeSymbol(_applyFn,2);
    *(t->nthArgument(0))=fn;
    *(t->nthArgument(1))=arg;
    return TermList(env.sharing->insert(t));
  }
  TermList getCurryFn(unsigned function)
  {
    if(!_constantTerms.find(function)) {
      Term* t = new(0) Term;
      t->makeSymbol(_curryConstants[function],0);
      _constantTerms.insert(function,TermList(env.sharing->insert(t)));
    }
    return _constantTerms.get(function);
  }

  Curryfier()
  : _curryConstants(32)
  {
    Signature* sig=env.signature;
    int funCnt=sig->functions();
    _curryConstants.ensure(funCnt);
    _constantTerms.ensure(funCnt);
    for(int i=0;i<funCnt;i++) {
      if(sig->functionArity(i)>0) {
	_curryConstants[i]=sig->addSkolemFunction(0);
      }
    }
    _applyFn=sig->addFunction("@",2);
  }

  unsigned _applyFn;
  DArray<unsigned> _curryConstants;
  ArrayMap<TermList> _constantTerms;
  DHMap<Term*,TermList, PtrIdentityHash > _termCache;
};

};

#endif
