
/*
 * File FormulaUnit.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include "Lib/Int.hpp"

#include "Formula.hpp"
#include "FormulaUnit.hpp"
#include "Inference.hpp"
#include "SubformulaIterator.hpp"
#include "Term.hpp"

using namespace std;
using namespace Lib;

using namespace Kernel;

void FormulaUnit::destroy()
{
  _inference->destroy();
  delete this;
} 


string FormulaUnit::toString() const
{
  return Int::toString(_number) + ". " + _formula->toString() +
         ' ' + inferenceAsString();
} 

unsigned FormulaUnit::varCnt()
{
  CALL("FormulaUnit::varCnt");

  Formula* frm = formula();
  Formula::VarList* fv = frm->freeVariables();
  Formula::VarList* bv = frm->boundVariables();

  unsigned res = fv->length() + bv->length();
  fv->destroy();
  bv->destroy();
  return res;
}

Color FormulaUnit::getColor()
{
  CALL("FormulaUnit::getColor");
  ASS_ALLOC_TYPE(this, "FormulaUnit");

  return this->formula()->getColor();
}
