
/*
 * File Inference.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __Inference__
#define __Inference__

#include <cstdlib>
#include <string>

#include "Kernel/Unit.hpp"
#include "Lib/Allocator.hpp"

using namespace std;
using namespace Lib;

namespace Kernel {

class Inference
{
public:
 
  enum Rule {
   
    INPUT,
   
    NEGATED_CONJECTURE,
   
    ANSWER_LITERAL,






   
    RECTIFY,






















   
    CLOSURE,
   
    FLATTEN,
   
    FORMULA_ITE_EXPANSION,
   
    TERM_ITE_DEFINITION,
   
    REORDER_LITERALS,
   
    ENNF,
   
    NNF,




















   
    SKOLEMIZE,
   
    CLAUSIFY,
   
    REMOVE_DUPLICATE_LITERALS,


   
    RESOLUTION,
   
    EQUALITY_PROXY_REPLACEMENT,
   
    EQUALITY_PROXY_AXIOM1,
   
    EQUALITY_PROXY_AXIOM2,
   
    DEFINITION_UNFOLDING,
   
    DEFINITION_FOLDING,
   
    SKOLEM_PREDICATE_INTRODUCTION,
   
    PREDICATE_SKOLEMIZE,


   
    PREDICATE_DEFINITION,
   
    PREDICATE_DEFINITION_UNFOLDING,
   
    PREDICATE_DEFINITION_MERGING,
   
    EQUIVALENCE_DISCOVERY,
   
    FORMULA_SHARING,
   
    REDUCE_FALSE_TRUE,
   
    LOCAL_SIMPLIFICATION,
   
    NORMALIZATION,
   
    EQUALITY_PROPAGATION,
   
    TRIVIAL_INEQUALITY_REMOVAL,
   
    FACTORING,
   
    SUBSUMPTION_RESOLUTION,
   
    SUPERPOSITION,
   
    EQUALITY_FACTORING,
   
    EQUALITY_RESOLUTION,
   
    FORWARD_DEMODULATION,
   
    BACKWARD_DEMODULATION,
   
    FORWARD_LITERAL_REWRITING,
   
    CONDENSATION,
   
    EVALUATION,
   
    INTERPRETED_SIMPLIFICATION,
   
    UNUSED_PREDICATE_DEFINITION_REMOVAL,
   
    PURE_PREDICATE_REMOVAL,
   
    INEQUALITY_SPLITTING,
   
    INEQUALITY_SPLITTING_NAME_INTRODUCTION,
   
    GROUNDING,
   
    EQUALITY_AXIOM,
   
    THEORY,
   
    BOOLEAN_TERM_ENCODING,
   
    TERM_IF_THEN_ELSE_DEFINITION,
   
    SPECIAL_TERM_ELIMINATION,
   
    SPLITTING,
   
    SPLITTING_COMPONENT,
   
    BACKTRACKING_SPLITTING_COMPONENT,
   
    BACKTRACKING_SPLIT_REFUTATION,
   
    GENERAL_SPLITTING,
   
    GENERAL_SPLITTING_COMPONENT,
   
    COMMON_NONPROP_MERGE,
   
    PROP_REDUCE,
   
    CLAUSE_NAMING,
   
    BDDZATION,
   
    TAUTOLOGY_INTRODUCTION,
   
    PROGRAM_ANALYSIS,
   
    COLOR_UNBLOCKING,
   
    INSTANCE_GENERATION,
   
    UNIT_RESULTING_RESOLUTION,
   
    HYPER_SUPERPOSITION,
   
    GLOBAL_SUBSUMPTION,
   
    DISTINCT_EQUALITY_REMOVAL,
   
    EXTERNAL,
   
    CLAIM_DEFINITION,
   
    BFNT_FLATTENING,
   
    BFNT_DISTINCT,
   
    BFNT_TOTALITY,
  }; 

  explicit Inference(Rule r);

 
  virtual void destroy();
 
  virtual ~Inference() {}

  static string ruleName(Rule rule);
  string name() const { return ruleName(_rule); }

  CLASS_NAME(Inference);
  USE_ALLOCATOR(Inference);

 
  struct Iterator {
   
    union {
      void* pointer;
      int integer;
    };
  };

  virtual Iterator iterator();
  virtual bool hasNext(Iterator& it);
  virtual Unit* next(Iterator& it);

 
  Rule rule() const { return _rule; }

protected:
 
  Rule _rule;
}; 

class Inference1
  : public Inference
{
public:
  Inference1(Rule rule,Unit* premise)
    : Inference(rule),
      _premise1(premise)
  { _premise1->incRefCnt(); }

  virtual void destroy();
  virtual Iterator iterator();
  virtual bool hasNext(Iterator& it);
  virtual Unit* next(Iterator& it);

  CLASS_NAME(Inference1);
  USE_ALLOCATOR(Inference1);

protected:
 
  Unit* _premise1;
};

class InferenceMany
  : public Inference
{
public:
  InferenceMany(Rule rule,UnitList* premises);
  virtual ~InferenceMany() { _premises->destroy(); }

  virtual void destroy();
  virtual Iterator iterator();
  virtual bool hasNext(Iterator& it);
  virtual Unit* next(Iterator& it);

  CLASS_NAME(InferenceMany);
  USE_ALLOCATOR(InferenceMany);

protected:
 
  UnitList* _premises;
};

class Inference2
  : public Inference
{
public:
  Inference2(Rule rule,Unit* premise1,Unit* premise2)
    : Inference(rule),
      _premise1(premise1),
      _premise2(premise2)
  {
    _premise1->incRefCnt();
    _premise2->incRefCnt();
  }

  virtual void destroy();
  virtual Iterator iterator();
  virtual bool hasNext(Iterator& it);
  virtual Unit* next(Iterator& it);

  CLASS_NAME(Inference2);
  USE_ALLOCATOR(Inference2);

protected:
 
  Unit* _premise1;
 
  Unit* _premise2;
};


}
#endif
