
/*
 * File KBO.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __KBO__
#define __KBO__

#include "Forwards.hpp"

#include "Lib/DArray.hpp"

#include "Ordering.hpp"

namespace Kernel {

using namespace Lib;

class KBOBase
: public Ordering
{
public:
  virtual Comparison compareFunctors(unsigned fun1, unsigned fun2) const;

protected:
  KBOBase(Problem& prb, const Options& opt);

  Result compareFunctionPrecedences(unsigned fun1, unsigned fun2) const;

  int predicatePrecedence(unsigned pred) const;
  int predicateLevel(unsigned pred) const;

 
  unsigned _predicates;
 
  unsigned _functions;
 
  DArray<int> _predicateLevels;
 
  DArray<int> _predicatePrecedences;
 
  DArray<int> _functionPrecedences;

  bool _reverseLCM;
};

class KBO
: public KBOBase
{
public:
  KBO(Problem& prb, const Options& opt);
  virtual ~KBO();

  virtual Result compare(Literal* l1, Literal* l2) const;
  virtual Result compare(TermList tl1, TermList tl2) const;
protected:

  class State;
 
  int _variableWeight;
 
  int _defaultSymbolWeight;

  int functionSymbolWeight(unsigned fun) const;

  bool allConstantsHeavierThanVariables() const { return false; }
  bool existsZeroWeightUnaryFunction() const { return false; }


 
  mutable State* _state;
};

}
#endif
