
/*
 * File MainLoop.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __MainLoop__
#define __MainLoop__

#include "Forwards.hpp"

#include "Lib/Environment.hpp"
#include "Lib/Exception.hpp"

#include "Shell/Statistics.hpp"

namespace Shell {
  class Property;
};

namespace Kernel {

using namespace Lib;
using namespace Inferences;
using namespace Shell;

struct MainLoopResult
{
  typedef Statistics::TerminationReason TerminationReason;

  MainLoopResult(TerminationReason reason)
  : terminationReason(reason), saturatedSet(0) {}
  MainLoopResult(TerminationReason reason, Clause* ref)
  : terminationReason(reason), refutation(ref), saturatedSet(0) {}

  void updateStatistics();

  TerminationReason terminationReason;
  Clause* refutation;
  UnitList* saturatedSet;
};



class MainLoop {
public:
  MainLoop(Problem& prb, const Options& opt) : _prb(prb), _opt(opt) {}
  virtual ~MainLoop() {}


  MainLoopResult run();
  static MainLoop* createFromOptions(Problem& prb, const Options& opt);

 
  struct RefutationFoundException : public ThrowableBase
  {
    RefutationFoundException(Clause* ref) : refutation(ref)
    {
      CALL("MainLoop::RefutationFoundException::RefutationFoundException");
      ASS(isRefutation(ref));
    }

    Clause* refutation;
  };

 
  struct MainLoopFinishedException : public ThrowableBase
  {
    MainLoopFinishedException(const MainLoopResult& res) : result(res)
    {
    }

    MainLoopResult result;
  };

 
  const Problem& getProblem() const { return _prb; }
  Problem& getProblem() { return _prb; }

 
  const Options& getOptions() const { return _opt; }

protected:
  static bool isRefutation(Clause* cl);
  static ImmediateSimplificationEngine* createISE(Problem& prb, const Options& opt);

 
  virtual void init() = 0;

 
  virtual MainLoopResult runImpl() = 0;

  Problem& _prb;

 
  const Options& _opt;
};

}

#endif 
