
/*
 * File MatchTag.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __MatchTag__
#define __MatchTag__

#include "Forwards.hpp"

#include "Lib/BitUtils.hpp"


#define USE_MATCH_TAG 0

namespace Kernel {

#if USE_MATCH_TAG


class MatchTag
{
public:
  inline void makeEmpty() throw() {_content=EMPTY_CONTENT; }

  inline bool isEmpty() {
    return _content==EMPTY_CONTENT;
  }

  inline void ensureInit(Term* t)
  {
    if(isEmpty()) {
      init(t);
    }
  }

  inline bool couldBeInstanceOf(MatchTag base)
  {
    if(base._content==EMPTY_CONTENT_REPLACEMENT) {
      return BitUtils::isSubset(EMPTY_CONTENT, _content);
    }
    return BitUtils::isSubset(base._content, _content);
  }
  inline bool couldBeInstanceOfReversed(MatchTag base)
  {
    if(base._content==EMPTY_CONTENT_REPLACEMENT) {
      return BitUtils::isSubset(EMPTY_CONTENT, _highContent) &&
        BitUtils::isSubset(base._highContent, _lowContent);
    }
    return BitUtils::isSubset(base._lowContent, _highContent) &&
      BitUtils::isSubset(base._highContent, _lowContent);
  }


  static const unsigned EMPTY_CONTENT=3;
  static const unsigned EMPTY_CONTENT_REPLACEMENT=1;
  static const unsigned CONTENT_BITS=sizeof(unsigned)*8;

private:
  void init(Term* t);

  static unsigned getContent(Term* t);

  union {
    unsigned _content;
    struct {
      unsigned _lowContent : CONTENT_BITS/2;
      unsigned _highContent : CONTENT_BITS/2;
    };
  };
};


ASS_STATIC( (MatchTag::EMPTY_CONTENT&MatchTag::EMPTY_CONTENT_REPLACEMENT)==MatchTag::EMPTY_CONTENT_REPLACEMENT );

ASS_STATIC( MatchTag::EMPTY_CONTENT<0x10000 );


#endif

};

#endif
