
/*
 * File RCClauseStack.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __RCClauseStack__
#define __RCClauseStack__

#include "Forwards.hpp"

#include "Lib/Stack.hpp"

#include "Kernel/Clause.hpp"

namespace Kernel {

using namespace Lib;

class RCClauseStack {
public:
  void push(Clause* cl)
  {
    CALL("RCClauseStack::push");

    cl->incRefCnt();
    _s.push(cl);
  }

  void pushWithoutInc(Clause* cl)
  {
    CALL("RCClauseStack::push");

    _s.push(cl);
  }

  Clause* pop()
  {
    CALL("RCClauseStack::pop");

    Clause* cl=_s.pop();
    cl->decRefCnt();
    return cl;
  }

 
  Clause* popWithoutDec()
  {
    CALL("RCClauseStack::popWithoutDec");

    return _s.pop();
  }

  bool isEmpty() { return _s.isEmpty(); }
  bool isNonEmpty() { return _s.isNonEmpty(); }

  void reset()
  {
    while(isNonEmpty()) {
      pop();
    }
  }

  size_t size() const
  {
    return _s.size();
  }

  class Iterator
  {
  public:
    DECL_ELEMENT_TYPE(Clause*);

    Iterator(const RCClauseStack& s) : _inner(s._s) {}

    bool hasNext() { return _inner.hasNext(); }
    Clause* next() { return _inner.next(); }

  private:
    ClauseStack::ConstIterator _inner;
  };

  class DelIterator
  {
  public:
    DECL_ELEMENT_TYPE(Clause*);

    DelIterator(RCClauseStack& s) : _inner(s._s) {}

    bool hasNext() { return _inner.hasNext(); }
    Clause* next() {
      curr = _inner.next();
      return curr;
    }
    void del() {
      _inner.del();
      curr->decRefCnt();
    }
    void replace(Clause* replacement) {
      _inner.replace(replacement);
      replacement->incRefCnt();
      curr->decRefCnt();
      curr = replacement;
    }

  private:
    ClauseStack::DelIterator _inner;
    Clause* curr;
  };

private:
  ClauseStack _s;
};

}

#endif 
