
/*
 * File BitUtils.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __BitUtils__
#define __BitUtils__

#include <string.h>

#include "Lib/Portability.hpp"

namespace Lib {

class BitUtils
{
public:
 
  static bool memEqual(const void* ptr1, const void* ptr2, size_t sz)
  {
    return !memcmp(ptr1,ptr2,sz);
  }

 
  static void zeroMemory(void* ptr, size_t bytes)
  {
    size_t* sptr=static_cast<size_t*>(ptr);
    while(bytes>=sizeof(sptr)) {
      *(sptr++)=0;
      bytes-=sizeof(sptr);
    }
    char* cptr=reinterpret_cast<char*>(sptr);
    while(bytes) {
      *(cptr++)=0;
      bytes--;
    }
  }

 
  inline
  static bool getBitValue(void* ptr, size_t index)
  {
    unsigned char* cptr=static_cast<unsigned char*>(ptr)+index/8;
    return ((*cptr)>>(index&7))&1;
  }

 
  inline
  static void setBitValue(void* ptr, size_t index, bool value)
  {
    unsigned char* cptr=static_cast<unsigned char*>(ptr)+index/8;
    if(value) {
      *cptr|=1<<(index&7);
    } else {
      *cptr&=~(1<<(index&7));
    }
  }

 
  template<typename T>
  inline static bool isSubset(T set, T subset)
  {
    return (set&subset)==subset;
  }

 
  static unsigned reverseBits(unsigned v) __attribute__((const))
  {
    ASS_STATIC(sizeof(unsigned)==4);

    
    v = ((v >> 1) & 0x55555555) | ((v & 0x55555555) << 1);
    
    v = ((v >> 2) & 0x33333333) | ((v & 0x33333333) << 2);
    
    v = ((v >> 4) & 0x0F0F0F0F) | ((v & 0x0F0F0F0F) << 4);
    
    v = ((v >> 8) & 0x00FF00FF) | ((v & 0x00FF00FF) << 8);
    
    v = ( v >> 16             ) | ( v               << 16);
    return v;
  }

 
  static unsigned log2(unsigned v) __attribute__((const))
  {
    const unsigned int b[] = {0x2, 0xC, 0xF0, 0xFF00, 0xFFFF0000};
    const unsigned int S[] = {1, 2, 4, 8, 16};
    int i;

    register unsigned int r = 0; 
    for (i = 4; i >= 0; i--) 
    {
      if (v & b[i])
      {
        v >>= S[i];
        r |= S[i];
      }
    }
    return r;
  }

};

};

#endif
