
/*
 * File Environment.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __Environment__
#define __Environment__

#include <iostream>

#include "Forwards.hpp"
#include "Exception.hpp"

namespace Lib {

using namespace std;
using namespace Sys;

class Environment
{
public:
  Environment();
  ~Environment();

 
  Shell::Options* options;
 
  Kernel::Sorts* sorts;
 
  Kernel::Signature* signature;
 
  Indexing::TermSharing* sharing;
 
  Shell::Statistics* statistics;
 
  Shell::Property* property;
 
  Timer* timer;

  bool haveOutput();
  void beginOutput();
  void endOutput();
  ostream& out();

  void setPipeOutput(SyncPipe* pipe);
  SyncPipe* getOutputPipe() { return _pipe; }

  void setPriorityOutput(ostream* stm);
  ostream* getPriorityOutput() { return _priorityOutput; }

  bool timeLimitReached() const;

  template<int Period>
  void checkTimeSometime() const
  {
    static int counter=0;
    counter++;
    if(counter==Period) {
      counter=0;
      if(timeLimitReached()) {
        throw TimeLimitExceededException();
      }
    }
  }
 
  int remainingTime() const;
 
  Kernel::Ordering* ordering;
 
  bool colorUsed;
 
  bool interpretedOperationsUsed;

private:
  int _outputDepth;
 
  ostream* _priorityOutput;
  SyncPipe* _pipe;
}; 

extern Environment env;

}

#endif




