
/*
 * File Graph.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __IntGraph__
#define __IntGraph__

#include <utility>

#include "DArray.hpp"
#include "DHMultiset.hpp"
#include "Numbering.hpp"
#include "Stack.hpp"

namespace Lib {

using namespace std;

class IntDiGraph
{
public:
  IntDiGraph() : _nodeCnt(0) {}
  unsigned addNode();
  void addEdge(unsigned src, unsigned dest);

 
  unsigned edge(unsigned src, unsigned dest);

 
  void neighbours(unsigned node, unsigned*& neighArrPtr, unsigned& neighCnt);

  unsigned size() const { return _nodeCnt; }

private:
  unsigned _nodeCnt;
  DArray<Stack<unsigned> > _neighs;
  DHMultiset<pair<unsigned,unsigned> > _edges;
};

class IntGraph : IntDiGraph
{
public:
  unsigned addNode()
  {
    return IntDiGraph::addNode();
  }
  void addEgde(unsigned src, unsigned dest)
  {
    IntDiGraph::addEdge(src, dest);
    if(src!=dest) {
      IntDiGraph::addEdge(dest, src);
    }
  }
  unsigned edge(unsigned src, unsigned dest)
  {
    ASS_EQ(IntDiGraph::edge(src, dest), IntDiGraph::edge(dest, src));
    return IntDiGraph::edge(src, dest);
  }
  void neighbours(unsigned node, unsigned*& neighArrPtr, unsigned& neighCnt)
  {
    IntDiGraph::neighbours(node, neighArrPtr, neighCnt);
  }
};

class IntSubgraph
{
public:
  static IntSubgraph* getFull() { return 0; }
  static IntSubgraph* getEmpty();
private:
  DHMultiset<unsigned> _nodes;
};

template<typename T, class IGClass=IntGraph>
class Graph
{
public:
  void addNode(T obj)
  {
    unsigned inode=_ig.addNode();
    _num.assign(obj, inode);
  }
  void addEgde(T src, T dest)
  {
    _ig.addEdge( _num.get(src), _num.get(dest) );
  }
  unsigned edge(unsigned src, unsigned dest)
  {
    return _ig.edge( _num.get(src), _num.get(dest) );
  }
  bool contains(T obj)
  {
    return _num.contains(obj);
  }
private:
  Numbering<T> _num;
  IGClass _ig;
};

};

#endif
