
/*
 * File Hash.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include "Debug/Assertion.hpp"
#include "Debug/Tracer.hpp"

#include "Kernel/Unit.hpp"

#include "Portability.hpp"

#include "Hash.hpp"


namespace Lib
{

using namespace std;

unsigned Hash::hash(Kernel::Unit* u)
{
  if(!u) {
    return 75663234u; 
  }
  return hash(u->number());
}

unsigned Hash::hash (const char* val)
{
  CALL("Hash::hash/2");

  unsigned hash = 2166136261u;
  while (*val) {
    hash = (hash ^ *val) * 16777619u;
    val++;
  }
  return hash;
} 

unsigned Hash::hash (const unsigned char* val,size_t size)
{
  CALL("Hash::hash/1");
  ASS(size > 0);

  unsigned hash = 2166136261u;
  for (int i = size-1;i >= 0;i--) {
    hash = (hash ^ val[i]) * 16777619u;
  }
  return hash;
} 

unsigned Hash::hash (const unsigned char* val,size_t size,unsigned hash)
{
  CALL("Hash::hash/0");
  ASS(size > 0);

  for (int i = size-1;i >= 0;i--) {
    hash = (hash ^ val[i]) * 16777619u;
  }
  return hash;
} 

}
