
/*
 * File ImplicationSetClosure.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __ImplicationSetClosure__
#define __ImplicationSetClosure__

#include "Forwards.hpp"

#include "DHSet.hpp"
#include "MapToLIFO.hpp"
#include "Stack.hpp"



namespace Lib {

template<typename T>
class ImplicationSetClosure {
private:
  typedef DHSet<T> SelSet;
  typedef MapToLIFO<T,T> ImplMap;

  SelSet _selected;
  ImplMap _implied;
public:
  void add(const T& val) {
    CALL("ImplicationSetClosure::add");

    if(_selected.contains(val)) { return; }
    static Stack<T> todo;
    todo.reset();
    todo.push(val);
    while(todo.isNonEmpty()) {
      T v = todo.pop();
      _selected.insert(v);
      typename ImplMap::ValIterator implIt = _implied.keyIterator(v);
      while(implIt.hasNext()) {
	T impl = implIt.next();
	if(_selected.contains(impl)) { continue; }
	todo.push(impl);
      }
    }
  }

  template<class It>
  void addFromIterator(It it) {
    CALL("ImplicationSetClosure::addFromIterator");

    while(it.hasNext()) {
      add(it.next());
    }
  }

  void addImplication(const T& prem, const T& conseq) {
    CALL("ImplicationSetClosure::addImplication");

    if(prem==conseq) { return; }
    _implied.pushToKey(prem, conseq);
    if(_selected.contains(prem)) {
      add(conseq);
    }
  }

  bool contains(const T& val) const {
    return _selected.contains(val);
  }

  class Iterator : public SelSet::Iterator
  {
  public:
    Iterator(const ImplicationSetClosure& parent) : DHSet<T>::Iterator(parent._selected) {}
  };
};

}

#endif 
