
/*
 * File Int.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include <cstdio>
#include <cstdlib>
#include <cerrno>
#include <iostream>
#include <climits>

#include "Int.hpp"
#include "Debug/Tracer.hpp"

using namespace Lib;

int Int::gcd(int i,int j)
{
  CALL("Int::gcd");
  
  i=abs(i);
  j=abs(j);
  if(!i || !j) {
    return 1;
  }
  
  for(;;) {
    i = i % j;
    if(i==0) {
      return j;
    }
    j = j % i;
    if(j==0) {
      return i;
    }
  }
}

bool Int::safeUnaryMinus(int num, int& res)
{
  CALL("Int::safeUnaryMinus");
  
  if(num == INT_MIN) {
    return false;
  }
  res=-num;
  return true;
}

bool Int::safePlus(int arg1, int arg2, int& res)
{
  CALL("Int::safePlus");
  
  if(arg2<0) {
    if(INT_MIN - arg2 > arg1) { return false; }
  }
  else {
    if(INT_MAX - arg2 < arg1) { return false; }
  }
  res=arg1+arg2;
  return true;
}

bool Int::safeMinus(int num, int sub, int& res)
{
  CALL("Int::safeMinus");
  
  if(sub<0) {
    if(INT_MAX + sub < num) { return false; }
  }
  else {
    if(INT_MIN + sub > num) { return false; }
  }
  res=num-sub;
  return true;
}

bool Int::safeMultiply(int arg1, int arg2, int& res)
{
  CALL("Int::safeMultiply");
  
  ASS_STATIC(sizeof(int)==4);
  ASS_STATIC(sizeof(long long)==8);
  long long mres=static_cast<long long>(arg1)*arg2;
  if(mres>INT_MAX || mres<INT_MIN) {
    return false;
  }
  res=mres;
  return true;
}

string Int::toString (int i)
{
  char tmp [20];
  sprintf(tmp,"%d",i);
  string result(tmp);

  return result;
} 


string Int::toString(double d)
{
  char tmp [256];
  sprintf(tmp,"%g",d);
  string result(tmp);

  return result;
} 


string Int::toString(long l)
{
  char tmp [256];
  sprintf(tmp,"%ld",l);
  string result(tmp);

  return result;
} 


string Int::toString(unsigned i)
{
  char tmp [256];
  sprintf(tmp,"%d",i);
  string result(tmp);

  return result;
} 

string Int::toString(unsigned long i)
{
  char tmp [256];
  sprintf(tmp,"%lu",i);
  string result(tmp);

  return result;
} 

string Int::toHexString(size_t i)
{
  char tmp [256];
  sprintf(tmp,"0x%zx",i);
  string result(tmp);

  return result;
} 


bool Int::stringToLong (const char* str,long& result)
{
  CALL("Int::stringToLong");

  if (! *str) { 
    return false;
  }

  errno = 0;
  char* endptr = 0;
  result = strtol(str,&endptr,10);

  if (*endptr ||
      (result == 0 && errno) ||
      ( (result == LONG_MAX || result == LONG_MIN) && errno==ERANGE ) ) { 
    return false;
  }

  return true;
} 


bool Int::stringToInt (const string& str,int& result)
{
  CALL("Int::stringToInt");
  return stringToInt(str.c_str(),result);
} 

bool Int::stringToUnsignedInt (const string& str,unsigned& result)
{
  CALL("Int::stringToUnsignedInt");
  return stringToUnsignedInt(str.c_str(),result);
} 

bool Int::stringToUnsignedInt (const char* str,unsigned& result)
{
  CALL("Int::stringToUnsignedInt");

  int i;
  if (stringToInt(str,i) && i >= 0) {
    result = i;
    return true;
  }
  return false;
} 

bool Int::stringToInt (const char* str,int& result)
{
  long ln;
  bool converted = stringToLong(str,ln);
  if (! converted || ln > INT_MAX || ln < INT_MIN) {
    return false;
  }
  result = (int)ln;
  return true;
} 


bool Int::stringToDouble (const char* str,double& result)
{
  CALL("Int::stringToDouble");

  errno = 0;
  char* endptr = 0;
  result = strtod(str,&endptr);

  if (*endptr ||
      (result == 0.0 && errno)) { 
    return false;
  }

  return true;
} 


bool Int::stringToFloat (const char* str,float& result)
{
  double d;
  bool converted = stringToDouble(str,d);



  if (! converted) {
    return false;
  }
  result = (float)d;
  return true;
} 


bool Int::stringToUnsigned64 (const char* str,long long unsigned& result)
{
  result = 0;
  if (! *str) { 
    return false;
  }
  
  while (*str == '0') {
    str++;
  }
  while (*str) {
    char nextChar = *str;
    str++;
    if (nextChar < '0' || nextChar > '9') {
      return false;
    }
    result = 10*result + (nextChar - '0');
  }
  return true;
} 

bool Int::stringToUnsigned64 (const string& str,long long unsigned& result)
{
  return stringToUnsigned64(str.c_str(),result);
} 

bool Int::isInteger(const char* str)
{
  CALL("Int::isInteger");

	if (*str == '-') {
		str++;
	}

	
	if (! *str) {
		return false;
	}

	
	do {
		if (*str < '0' || *str > '9') {
			return false;
		}
		str++;
	}
	while (*str);

	return true;
} 

