
/*
 * File LastCopyWatcher.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __LastCopyWatcher__
#define __LastCopyWatcher__

#include "Debug/Assertion.hpp"

namespace Lib {

class LastCopyWatcher
{
public:
  LastCopyWatcher()
  {
    _next=_previous=this;
  }
  LastCopyWatcher(const LastCopyWatcher& obj)
  {
    connect(&obj);
  }
  ~LastCopyWatcher()
  {
    disconnect();
  }
  LastCopyWatcher& operator=(const LastCopyWatcher& obj)
  {
    disconnect();
    connect(&obj);
    return *this;
  }
  bool isLast()
  {
    return this==_next;
  }
private:
  void disconnect()
  {
    _next->_previous=_previous;
    _previous->_next=_next;
  }

  void connect(const LastCopyWatcher* obj)
  {
    ASS_EQ(obj->_next->_previous, obj);

    _next=obj->_next;
    _previous=obj;

    _next->_previous=this;
    obj->_next=this;
  }

  mutable const LastCopyWatcher* _next;
  mutable const LastCopyWatcher* _previous;
};

};

#endif
