
/*
 * File MultiColumnMap.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __MultiColumnMap__
#define __MultiColumnMap__

#include "Allocator.hpp"
#include "BitUtils.hpp"
#include "DHMap.hpp"

namespace Lib {

template<typename Key>
class MultiColumnMap
{
public:
  MultiColumnMap() : _entryCount(0), _entryDataSize(0) {}

  ~MultiColumnMap()
  {
    
    typename DHMap<Key,char*>::Iterator dit(_data);
    while(dit.hasNext()) {
      char* entry=dit.next();
      DEALLOC_KNOWN(entry, getEntrySize(), "MultiColumnMap");
    }
  }

  template<typename T>
  class AccessObject;

  template<typename T>
  AccessObject<T> addColumn()
  {
    
    ASS_EQ(_data.size(),0);

    AccessObject<T> res(this, _entryCount, _entryDataSize);
    _entryCount++;
    _entryDataSize+=sizeof(T);
    return res;
  }

  template<typename T>
  class AccessObject
  {
  public:
    bool getOrCreate(Key k,T*& ptr)
    {
      char* entry=_obj->getEntry(k);
      bool res=!BitUtils::getBitValue(entry+_entryDataSize, _index);
      ptr=entry+_offset;
      return res;
    }
  private:
    AccessObject(MultiColumnMap* obj, unsigned index, unsigned offset)
    : _obj(obj), _index(index), _offset(offset) {}

    MultiColumnMap* _obj;
    unsigned _index;
    unsigned _offset;

    friend class MultiColumnMap;
  };

private:
  char* getEntry(Key k)
  {
    char** ppdata;
    if(_data.getValuePtr(k, ppdata)) {
      *ppdata=ALLOC_KNOWN(getEntrySize(), "MultiColumnMap");
      BitUtils::zeroMemory(*ppdata+_entryDataSize, getOccupancyBitsSize());
    }
    return *ppdata;
  }

  inline
  unsigned getOccupancyBitsSize()
  { return ((_entryCount-1)/8+1); }

  inline
  unsigned getEntrySize()
  { return _entryDataSize + getOccupancyBitsSize(); }

  unsigned _entryCount;
  unsigned _entryDataSize;
  DHMap<Key,char*> _data;
};

};

#endif
