
/*
 * File NameArray.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include <cstring>

#include "Exception.hpp"
#include "NameArray.hpp"
#include "Debug/Tracer.hpp"

namespace Lib {

NameArray::NameArray (const char* array[],int len)
  : length(len),
    _array(array)

{
#if VDEBUG
  for(int i=1; i<len; i++) {
    ASS_REP2(strcmp(array[i-1],array[i])<0,array[i-1],array[i]);
  }
#endif
} 

int NameArray::find (const char* value) const
{
  CALL("NameArray::find");

  int res=tryToFind(value);
  if(res==-1) {
    throw ValueNotFoundException();
  }
  return res;
} 

int NameArray::tryToFind(const char* value) const
{
  CALL("NameArray::tryToFind");

  int from = 0;
  int to = length;
  while (from < to) {
    int mid = (from+to)/2;
    int comp = strcmp(_array[mid],value);
    if (comp < 0) {
      from = mid+1;
    }
    else if (comp == 0) {
      return mid;
    }
    else { 
      to = mid;
    }
  }
  return -1;
}

} 
