
/*
 * File Numbering.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __Numbering__
#define __Numbering__

#include "DHMap.hpp"

namespace Lib {

template<typename T, unsigned Start=0>
class Numbering
{
public:
  Numbering() : _nextNum(Start) {}
  unsigned get(T obj)
  {
    CALL("Numbering::get");
    unsigned* pres;
    if(_map.getValuePtr(obj, pres, _nextNum)) {
      ALWAYS(_rev.insert(_nextNum, obj));
      _nextNum++;
    }
    return *pres;
  }
  void assign(T obj, unsigned num)
  {
    CALL("Numbering::assign");
    if(_map.insert(obj, num)) {
      ALWAYS(_rev.insert(num, obj));
      if(num>=_nextNum) {
        _nextNum=num+1;
      }
    }
#if VDEBUG
    else {
      ASS_EQ(_map.get(obj),num);
    }
#endif
  }
 
  unsigned getSpareNum()
  {
    CALL("Numbering::getSpareNum");
    return _nextNum++;
  }
  T obj(unsigned num) const
  {
    CALL("Numbering::obj");
    return _rev.get(num);
  }
  bool findObj(unsigned num, T& res) const
  {
    return _rev.find(num, res);
  }
  bool contains(T obj) const
  {
    return _map.find(obj);
  }
 
  unsigned getNumberUpperBound() const
  {
    CALL("Numbering::getNumberUpperBound");
    return _nextNum==0 ? 0 : (_nextNum-1);
  }
private:
  DHMap<T, unsigned> _map;
  DHMap<unsigned, T> _rev;

  unsigned _nextNum;
};

};

#endif
