
/*
 * File Random.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __RANDOM__
#  define __RANDOM__


#include <cstdlib>
#include <ctime>

namespace Lib
{

class Random 
{
 
  static int _seed;
 
  static int _remainingBits;
 
  static const int _bitsPerInt; 
 
  static unsigned _bits; 

  static int bitsPerInt ();     

 public:

 
  static inline int getMax () { return RAND_MAX; }

 
  static inline int getInteger () { return rand(); }
 
  static inline int getInteger (int modulus)
  { return getInteger () % modulus; }

 
  static inline int getBit()
  {
    if ( _remainingBits == 0 ) {
      _remainingBits = _bitsPerInt;
      _bits = getInteger();
    }

    int result = _bits % 2;
    _bits /= 2;
    _remainingBits --;

    return result;
  } 

  
 
  inline static void setSeed(int s)
  {
    _seed = s;
    srand(s);
  }

 
  inline static int seed()
  {
    return _seed;
  }

 
  inline static void resetSeed ()
  { setSeed(static_cast<int>(time(0))); }
}; 


} 
#endif


