
/*
 * File ReferenceCounter.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __ReferenceCounter__
#define __ReferenceCounter__

#include "Allocator.hpp"
#include "Debug/Assertion.hpp"

namespace Lib {

class ReferenceCounter
{
public:
  ReferenceCounter()
  {
    _counter=ALLOC_KNOWN(sizeof(Counter), "ReferenceCounter::Counter");
    *_counter=1;
  }
  ReferenceCounter(const ReferenceCounter& rc)
  {
    _counter=rc._counter;
    inc();
  }
  ~ReferenceCounter()
  {
    dec();
  }
  ReferenceCounter& operator=(const ReferenceCounter& rc)
  {
    dec();
    _counter=rc._counter;
    inc();
    return *this;
  }
  bool isLast()
  {
    return !*_counter;
  }
private:
  inline void inc()
  {
    (*_counter)++;
    ASS_NEQ(*_counter,0);
  }
  inline void dec()
  {
    ASS_NEQ(*_counter,0);
    (*_counter)--;
    if(!*_counter) {
      DEALLOC_KNOWN(_counter,sizeof(Counter),"ReferenceCounter::Counter");
    }
  }

  typedef size_t Counter;
  Counter* _counter;
};

};

#endif
