
/*
 * File Semaphore.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __Semaphore__
#define __Semaphore__

#include <sys/types.h>

#include "Forwards.hpp"

#include "Lib/Portability.hpp"

namespace Lib {
namespace Sys {

#if !COMPILER_MSVC

class Semaphore
{
public:
  Semaphore() : semid(-1), semCnt(0) {};
  explicit Semaphore(int num);
  ~Semaphore();

  Semaphore(const Semaphore& s);
  const Semaphore& operator=(const Semaphore& s);

  bool hasSemaphore() { return semid!=-1; }

  void inc(int num);
  void dec(int num);
  int get(int num);
  void set(int num, int val);

  bool isLastInstance();

private:

  void doInc(int num);
  void doDec(int num);
  void doSet(int num, int val);
  int doGet(int num);

  void registerInstance(bool addToInstanceList=true);
  void deregisterInstance();

  void acquireInstance();
  void releaseInstance();

  int semid;
 
  int semCnt;

  static void releaseAllSemaphores();
  static void postForkInChild();
  static void ensureEventHandlersInstalled();

  typedef List<Semaphore*> SemaphoreList;

  static SemaphoreList* s_instances;

  union semun {
      int              val;   
      struct semid_ds *buf;   
      unsigned short  *array; 
      struct seminfo  *__buf; 
  };
};

#endif

}
}

#endif 
