
/*
 * File SyncPipe.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __SyncPipe__
#define __SyncPipe__

#include <fstream>

#include "Forwards.hpp"

#include "Lib/fdstream.hpp"
#include "Lib/Exception.hpp"
#include "Lib/Portability.hpp"

#include "Semaphore.hpp"

namespace Lib {
namespace Sys {

#if COMPILER_MSVC

class SyncPipe {
public:
  bool isReading() const { NOT_IMPLEMENTED; }
  bool canRead() const { NOT_IMPLEMENTED; }
  void acquireRead() { NOT_IMPLEMENTED; }
  void releaseRead() { NOT_IMPLEMENTED; }
  void neverRead() { NOT_IMPLEMENTED; }
  bool isWriting() const { NOT_IMPLEMENTED; }
  bool canWrite() const { NOT_IMPLEMENTED; }
  void acquireWrite() { NOT_IMPLEMENTED; }
  void releaseWrite() { NOT_IMPLEMENTED; }
  void neverWrite() { NOT_IMPLEMENTED; }
  void releasePriviledges() { NOT_IMPLEMENTED; }
  istream& in() { NOT_IMPLEMENTED; }
  ostream& out() { NOT_IMPLEMENTED; }
};

#else

class SyncPipe {
public:
  SyncPipe();
  ~SyncPipe();

 
  bool isReading() const { return _isReading; }
 
  bool canRead() const { return _istream; }

  void acquireRead();
  void releaseRead();
  void neverRead();

 
  bool isWriting() const { return _isWriting; }
 
  bool canWrite() const { return _ostream; }

  void acquireWrite();
  void releaseWrite();
  void neverWrite();

  void releasePriviledges();

 
  istream& in()
  {
    ASS(_istream);
    ASS(isReading());
    if(!isReading()) {
      INVALID_OPERATION("Unallowed read from pipe.");
    }
    return *_istream;
  }

 
  ostream& out()
  {
    ASS(_ostream);
    ASS(isWriting());
    if(!isWriting()) {
      INVALID_OPERATION("Unallowed write to pipe.");
    }
    return *_ostream;
  }

private:
  SyncPipe(const SyncPipe&); 
  const SyncPipe& operator=(const SyncPipe&); 

 
  fdstream *_istream;
 
  fdstream *_ostream;

  int _readDescriptor;
  int _writeDescriptor;
  bool _isReading;
  bool _isWriting;

 
  Semaphore _syncSemaphore;

  static void postForkChildHadler();
  static void terminationHadler();
  static void ensureEventHandlersInstalled();

  typedef List<SyncPipe*> PipeList;

  static PipeList* s_instances;
};

#endif 

}
}

#endif 
