
/*
 * File Timer.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __Timer__
#define __Timer__

#include <string>
#include <iostream>

#include "Debug/Assertion.hpp"

#ifndef UNIX_USE_SIGALRM

#define UNIX_USE_SIGALRM !VDEBUG
#endif


#ifdef VAPI_LIBRARY
#if VAPI_LIBRARY

#undef UNIX_USE_SIGALRM
#define UNIX_USE_SIGALRM 0

#endif
#endif

namespace Lib
{

using namespace std;

class Timer
{
public:
  Timer(bool mustIncludeChildren=false) :
    _mustIncludeChildren(mustIncludeChildren),
    _running(false),
    _elapsed(0)
  { ensureTimerInitialized(); }

 
  inline void reset()
  { _running = false;
    _elapsed = 0; }

 
  inline void stop()
  {
    ASS(_running);

    _elapsed += miliseconds() - _start;
    _running = false;
  }

 
  inline void start()
  {
    ASS(! _running);

    _running = true;
    _start = miliseconds();
  } 

 
  int elapsedSeconds()
  {
    return elapsed()/1000;
  }

 
  int elapsedDeciseconds()
  {
    return elapsed()/100;
  }

 
  int elapsedMilliseconds()
  {
    return elapsed();
  }

  void makeChildrenIncluded();

  static void ensureTimerInitialized();
  static string msToSecondsString(int ms);
  static void printMSString(ostream& str, int ms);

  static void setTimeLimitEnforcement(bool enabled)
  { s_timeLimitEnforcement = enabled; }

  static void syncClock();

  static bool s_timeLimitEnforcement;
private:
 
  bool _mustIncludeChildren;
 
  bool _running;
 
  int _start;
 
  int _elapsed;

  int miliseconds();

#if UNIX_USE_SIGALRM
  static void suspendTimerBeforeFork();
  static void restoreTimerAfterFork();

  static int guaranteedMilliseconds();

  static long s_ticksPerSec;
  static int s_initGuarantedMiliseconds;
#endif

 
  inline
  int elapsed()
  {
    return _running ? miliseconds() - _start + _elapsed : _elapsed;
  }
}; 

} 

#endif
