
/*
 * File ISSatSweeping.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __ISSatSweeping__
#define __ISSatSweeping__

#include "Forwards.hpp"

#include "Lib/ArrayMap.hpp"
#include "Lib/DHSet.hpp"
#include "Lib/IntUnionFind.hpp"
#include "Lib/Metaiterators.hpp"
#include "Lib/Stack.hpp"

#include "SATSolver.hpp"

namespace SAT {

using namespace Lib;

class ISSatSweeping
{
public:
 
  typedef pair<SATLiteral,SATLiteral> Impl;
  typedef pair<SATLiteral,SATLiteral> Equiv;

  ISSatSweeping(unsigned varCnt, SATSolver& solver,
      IntIterator interestingVarIterator = IntIterator::getInvalid(),
      bool doRandomSimulation = true, unsigned conflictLimit = UINT_MAX, bool collectImplications=true);


  const DHSet<Impl>& getImplications() const { return _implications; }
  const Stack<Equiv>& getEquivalences() const { return _equivStack; }
  const SATLiteralStack& getTrueLiterals() const { return _trueLits; }

  SATLiteral getCandidateLiteral(unsigned var) const { return SATLiteral(var, _candidateVarPolarities[var]); }
 
  const IntUnionFind& getEquivalenceClasses() const { return _equivalentVars; }
private:


  bool sameCandGroup(unsigned v1, unsigned v2);


  void splitByCurrAssignment(SATLiteralStack& orig, SATLiteralStack& newGrp);
  void splitGroupsByCurrAssignment();

  void addTrueLit(SATLiteral lit);




  void lookForImplications(SATLiteral probedLit, bool assignedOpposite,
      bool& foundEquivalence);
  void addImplication(Impl i, bool& foundEquivalence);
  void onRedundantImplicationDiscovered();
  void doRedundantImplicationSweeping();

  bool tryProvingImplicationInner(Impl imp, bool& foundEquivalence);
  bool tryProvingImplication(Impl imp, bool& foundEquivalence);

  void createCandidates();
  void tryRandomSimulation();

  bool getProbingCandidatesFromGroup(SATLiteralStack& grp, unsigned firstIndex, SATLiteral& cand1, SATLiteral& cand2);
  bool getProbingCandidatesWithinRotation(SATLiteral& cand1, SATLiteral& cand2);
  bool nextRotation();
  bool getProbingCandidates(SATLiteral& cand1, SATLiteral& cand2);

  bool doOneProbing();

  void run();

  
  bool _doRandomSimulation;
  bool _conflictUpperLimit;
  bool _collectImplications;


  unsigned _varCnt;
  Stack<unsigned> _interestingVars;
  ArraySet _interestingVarsSet;

 
  unsigned _probingGroupIndex;
  unsigned _probingElementIndex;

 
  unsigned _conflictCountLimit;

 
  bool _reachedUpperConflictLimit;

 
  DArray<bool> _candidateVarPolarities;
 
  Stack<SATLiteralStack> _candidateGroups;

 
  unsigned _unfinishedAmount;

 
  ArrayMap<unsigned> _candidateGroupIndexes;

  IntUnionFind _equivalentVars;

 
  DHSet<Impl> _implications;

 
  size_t _lastSweepingImplicationCnt;

  Stack<Equiv> _equivStack;

  DHSet<unsigned> _trueVarSet;
  SATLiteralStack _trueLits;

  SATSolver& _solver;
};

}

#endif 
