
/*
 * File MinimizingSolver.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __MinimizingSolver__
#define __MinimizingSolver__

#include "Forwards.hpp"

#include "Lib/DArray.hpp"
#include "Lib/DHMap.hpp"
#include "Lib/DHSet.hpp"
#include "Lib/ScopedPtr.hpp"
#include "Lib/Stack.hpp"

#include "SATSolver.hpp"



namespace SAT {

using namespace Lib;

class MinimizingSolver : public SATSolver {
public:
  MinimizingSolver(SATSolver* inner);

  virtual Status getStatus() { return _inner->getStatus(); }
  virtual SATClause* getRefutation() { return _inner->getRefutation(); }
  virtual bool hasAssumptions() const { return _inner->hasAssumptions(); }
  virtual void randomizeAssignment() { _inner->randomizeAssignment(); _assignmentValid = false; }


  virtual void addClauses(SATClauseIterator cit, bool onlyPropagate=false);
  virtual VarAssignment getAssignment(unsigned var);
  virtual bool isZeroImplied(unsigned var);
  virtual void collectZeroImplied(SATLiteralStack& acc) { _inner->collectZeroImplied(acc); }
  virtual SATClause* getZeroImpliedCertificate(unsigned var) { return _inner->getZeroImpliedCertificate(var); }

  virtual void ensureVarCnt(unsigned newVarCnt);

  virtual void addAssumption(SATLiteral lit, unsigned conflictCountLimit);
  virtual void retractAllAssumptions();

private:

  static bool isNonEmptyClause(SATClause* cl);

  SATLiteral getMostSatisfyingTrueLiteral();
  void selectLiteral(SATLiteral lit);

  bool tryPuttingToAnExistingWatch(SATClause* cl);
  void putIntoIndex(SATClause* cl);

  void processInnerAssignmentChanges();
  void processUnprocessed();
  void updateAssignment();


  unsigned _varCnt;
  SATSolverSCP _inner;

  DHMap<unsigned, bool> _assumptions;

 
  bool _assignmentValid;


 
  SATClauseStack _unprocessed;

 
  DArray<bool> _asgn;

 
  DArray<SATClauseStack> _watcher;

 
  DArray<unsigned> _unsClCnt;

 
  DArray<SATClauseStack> _clIdx;

  DHSet<SATClause*> _satisfiedClauses;

};

}

#endif 
