
/*
 * File RestartStrategy.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include <math.h>

#include "Debug/Assertion.hpp"
#include "Debug/Tracer.hpp"

#include "RestartStrategy.hpp"

namespace SAT
{

size_t RestartStrategy::increase(size_t val, float quotient)
{
  return static_cast<size_t>(ceilf(val*quotient));
}

size_t GeometricRestartStrategy::getNextConflictCount()
{
  CALL("GeometricRestartStrategy::getNextConflictCount");

  size_t res = _conflictCnt;
  _conflictCnt = increase(_conflictCnt,_increase);
  return res;
}


size_t LubyRestartStrategy::getLubyNumber(size_t i)
{
  CALL("LubyRestartStrategy::getLubyNumber");
  ASS_G(i,0);

 
  size_t k = 1;
  size_t power = 2;
  while (power < (i + 1)) {
      k += 1;
      power *= 2;
  }
  if (power == (i + 1)) {
    return (power / 2);
  }
  return (getLubyNumber(i - (power / 2) + 1));
}

size_t MinisatRestartStrategy::getNextConflictCount()
{
  CALL("MinisatRestartStrategy::getNextConflictCount");

  if(_innerCnt>=_outerCnt) {
    _outerCnt = increase(_outerCnt,_increase);
    _innerCnt = _initConflictCnt;
  }
  else {
    _innerCnt = increase(_innerCnt,_increase);
  }
  return _innerCnt;
}


}
