
/*
 * File SATClauseSharing.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include "Lib/Hash.hpp"

#include "SATClauseSharing.hpp"

namespace SAT
{

SATClause* SATClauseSharing::insert(SATClause* c)
{
  SATClause* res=_storage.insert(c);
  if(res!=c) {
    c->destroy();
  }
  return res;
}

void SATClauseSharing::wipe()
{
  ClauseSet::Iterator it(_storage);
  while(it.hasNext()) {
    SATClause* cl=it.next();
    if(!cl->kept()) {
      cl->destroy();
    }
  }
  _storage.~ClauseSet();
  ::new(&_storage) ClauseSet();
}


SATClauseSharing* SATClauseSharing::getInstance()
{
  static SATClauseSharing* inst=0;
  if(!inst) {
    inst=new SATClauseSharing();
  }
  return inst;
}


unsigned SATClauseSharing::Hasher::hash(SATClause* c)
{
  return Hash::hash(reinterpret_cast<const unsigned char*>(c->literals()),
	  c->length()*sizeof(SATLiteral));
}

bool SATClauseSharing::Hasher::equals(SATClause* c1, SATClause* c2)
{
  if(c1->length()!=c2->length()) {
    return false;
  }
  for(int i=c1->length();i>=0;i--) {
    if((*c1)[i]!=(*c2)[i]) {
      return false;
    }
  }
  return true;
}


}
