
/*
 * File SATSolver.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __SATSolver__
#define __SATSolver__

#include "Lib/MaybeBool.hpp"

#include "SATLiteral.hpp"

namespace SAT {

class SATSolver {
public:
  enum VarAssignment {
    TRUE,
    FALSE,
    DONT_CARE,
    NOT_KNOWN
  };
  enum Status {
    SATISFIABLE,
    UNSATISFIABLE,
   
    UNKNOWN
  };

  virtual ~SATSolver() {}

 
  virtual void addClauses(SATClauseIterator cit, bool onlyPropagate=false) = 0;
  virtual Status getStatus() = 0;
 
  virtual VarAssignment getAssignment(unsigned var) = 0;

 
  virtual void randomizeAssignment() = 0;

 
  virtual bool isZeroImplied(unsigned var) = 0;
 
  virtual void collectZeroImplied(SATLiteralStack& acc) = 0;
 
  virtual SATClause* getZeroImpliedCertificate(unsigned var) = 0;

  virtual void ensureVarCnt(unsigned newVarCnt) {}

  void addAssumption(SATLiteral lit, bool onlyPropagate=false) {
    CALL("SATSolver::addAssumption(SATLiteral,bool)");
    addAssumption(lit, onlyPropagate ? 0 : UINT_MAX);
  }
 
  virtual void addAssumption(SATLiteral lit, unsigned conflictCountLimit) = 0;
  virtual void retractAllAssumptions() = 0;
  virtual bool hasAssumptions() const = 0;

  virtual SATClause* getRefutation() = 0;

 
  bool trueInAssignment(SATLiteral lit)
  {
    CALL("SATSolver::trueInAssignment");

    VarAssignment asgn = getAssignment(lit.var());
    VarAssignment desired = lit.polarity() ? TRUE : FALSE;
    return asgn==desired;
  }

 
  bool falseInAssignment(SATLiteral lit)
  {
    CALL("SATSolver::trueInAssignment");

    VarAssignment asgn = getAssignment(lit.var());
    VarAssignment desired = lit.polarity() ? FALSE: TRUE;
    return asgn==desired;
  }
};

}

#endif 
