
/*
 * File TransparentSolver.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __TransparentSolver__
#define __TransparentSolver__

#include "Forwards.hpp"

#include "Lib/DArray.hpp"
#include "Lib/ScopedPtr.hpp"
#include "Lib/Stack.hpp"

#include "SATSolver.hpp"


namespace SAT {

class TransparentSolver : public SATSolver
{
public:
  TransparentSolver(SATSolver* inner);

  virtual Status getStatus() { return _inner->getStatus(); }
  virtual SATClause* getRefutation() { return _inner->getRefutation(); }
  virtual void randomizeAssignment() { _inner->randomizeAssignment(); }

  virtual void ensureVarCnt(unsigned newVarCnt);
  virtual void addClauses(SATClauseIterator cit, bool onlyPropagate);
  virtual VarAssignment getAssignment(unsigned var);
  virtual bool isZeroImplied(unsigned var) { return _inner->isZeroImplied(var); } 
  virtual SATClause* getZeroImpliedCertificate(unsigned var) { return _inner->getZeroImpliedCertificate(var); }
  virtual void collectZeroImplied(SATLiteralStack& acc) { _inner->collectZeroImplied(acc); }

  virtual void addAssumption(SATLiteral lit, unsigned conflictCountLimit);
  virtual void retractAllAssumptions();
  virtual bool hasAssumptions() const { return _assumptions.isNonEmpty(); }
private:

  void processUnprocessed();

  void processUnit(SATClause* cl);

  void makeVarNonPure(unsigned var);

  bool tryWatchOrSubsume(SATClause* cl, unsigned forbiddenVar=0);

  bool tryToSweepPure(unsigned var, bool eager);

  void flushClausesToInner(bool onlyPropagate);

  void addInnerAssumption(SATLiteral lit, unsigned conflictCountLimit);

  struct VarInfo
  {
    VarInfo() : _unseen(true), _isRewritten(false), _unit(0), _hasAssumption(false) {}

   
    bool _unseen;
    bool _isPure;
    bool _isRewritten;
   
    SATClause* _unit;
   
    bool _isPurePositive;
   
    SATClauseStack _watched;
   
    SATLiteral _root;

    bool _hasAssumption;
    bool _assumedPolarity;
  };

  SATSolverSCP _inner;

  
  SATClauseStack _unprocessed;
  SATClauseStack _toBeAdded;

  DArray<VarInfo> _vars;

  SATLiteralStack _assumptions;
};

}

#endif 
