
/*
 * File BSplitter.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __BSplitter__
#define __BSplitter__

#include "Forwards.hpp"

#include "Lib/Allocator.hpp"
#include "Lib/Array.hpp"
#include "Lib/Stack.hpp"

#include "Kernel/Clause.hpp"
#include "Kernel/RCClauseStack.hpp"

#include "Splitter.hpp"

namespace Saturation {

using namespace Kernel;

class BSplitter : public Splitter {
private:
  struct ReductionRecord
  {
    ReductionRecord(unsigned timestamp, Clause* clause) : timestamp(timestamp), clause(clause) {}
    unsigned timestamp;
    Clause* clause;
  };

  typedef Stack<SplitLevel> LevelStack;
  struct SplitRecord
  {
    SplitRecord(Clause* base, Clause* comp)
     : base(base), component(comp)
    {
      base->incRefCnt();
      component->incRefCnt();
    }

    ~SplitRecord()
    {
      base->decRefCnt();
      component->decRefCnt();
    }

    void addReduced(Clause* cl);

    Clause* base;
    Clause* component;
    LevelStack dependent;
    ClauseStack children;
    Stack<ReductionRecord> reduced;

    CLASS_NAME(BSplitter::SplitRecord);
    USE_ALLOCATOR(SplitRecord);
  };
public:
  BSplitter();

  virtual void init(SaturationAlgorithm* sa);

  bool doSplitting(Clause* cl);

  void onClauseReduction(Clause* cl, ClauseIterator premises, Clause* replacement);
  void onNewClause(Clause* cl);
  void onAllProcessed();

  bool handleEmptyClause(Clause* cl);

  void backtrack(ClauseIterator emptyClauses);
private:

  SplitSet* getTransitivelyDependentLevels(SplitLevel l);

  bool stackSplitting() { return false; }

  bool canBeSplitted(Clause* cl) { return true; }
  Clause* getComponent(Clause* icl);

  SplitSet* getNewClauseSplitSet(Clause* cl);
  void assignClauseSplitSet(Clause* cl, SplitSet* splits);


  Inference* getAlternativeClauseInference(Clause* base, Clause* firstComp, Clause* refutation);

  void getAlternativeClauses(Clause* base, Clause* firstComp, Clause* refutation, SplitLevel refLvl,
      RCClauseStack& acc, SplitSet*& altSplitSet);

  void assertSplitLevelsExist(SplitSet* s);

  bool _addGroundNegation;
  SplitLevel _nextLev;

  ZIArray<SplitRecord*> _db;
  RCClauseStack _splitRefutations;
};

}

#endif 
