
/*
 * File Limits.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __Limits__
#define __Limits__

#include "Forwards.hpp"

#include "Lib/Event.hpp"

#include "Kernel/Clause.hpp"


namespace Saturation
{

using namespace Lib;
using namespace Kernel;
using namespace Shell;

enum LimitsChangeType {
  NO_LIMITS_CHANGE=0,
  LIMITS_TIGHTENED=1,
  LIMITS_LOOSENED=2,
  GENERAL_LIMITS_CHANGE=3
};

typedef SingleParamEvent<LimitsChangeType> LimitsChangeEvent;

class Limits
{
public:
  Limits(const Options& opt) : _maxAge(-1), _maxWeight(-1), _opt(opt) {}

  LimitsChangeEvent changedEvent;

  int ageLimit() { return _maxAge; }
  bool ageLimited() { return _maxAge!=-1; }

  int weightLimit() { return _maxWeight; }
  int nonGoalWeightLimit() { return _maxNonGoalWeight; }
  bool weightLimited() { return _maxWeight!=-1; }

  bool fulfillsLimits(Clause* cl);

  void setLimits(int newMaxAge, int newMaxWeight);

private:
  int _maxAge;
  int _maxWeight;
  int _maxNonGoalWeight;
  const Options& _opt;
};

};

#endif
