
/*
 * File SWBSplitterWithoutBDDs.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __SWBSplitterWithoutBDDs__
#define __SWBSplitterWithoutBDDs__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"

#include "SWBSplitter.hpp"

namespace Saturation {

using namespace Lib;
using namespace Kernel;
using namespace Indexing;

class SWBSplitterWithoutBDDs : public SWBSplitter
{
protected:
  void buildAndInsertComponents(Clause* cl, const CompRec* comps, unsigned compCnt, bool firstIsMaster);

  
  virtual bool canStandAlone(Literal* lit);
  
  virtual bool standAloneObligations();

private:

  struct CompNameRec
  {
    CompNameRec() : name(0), namingClause(0) {}
    CompNameRec(int name, Clause* namingClause) : name(name), namingClause(namingClause) {}
    bool isEmpty() { return name==0; }

    int name;
    Clause* namingClause;
  };

  CompNameRec getNamedComponent(Clause* cl, const CompRec& cr);
  CompNameRec createNamedComponent(Clause* cl, const CompRec& cr, int knownName=0);

  Literal* getNameLiteral(int name, bool forMaster);
  int getNewName();

 
  DHMap<Literal*, CompNameRec> _groundNames;
};

};

#endif
