
/*
 * File Splitter.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __Splitter__
#define __Splitter__

#include "Forwards.hpp"

#include "Lib/Stack.hpp"

namespace Saturation {

using namespace Lib;
using namespace Kernel;
using namespace Indexing;
using namespace Shell;

class Splitter
{
public:
  virtual ~Splitter() {}

  virtual void init(SaturationAlgorithm* sa);

  virtual bool doSplitting(Clause* cl) = 0;

 
  virtual bool handleEmptyClause(Clause* cl) { return false; }

  virtual void onClauseReduction(Clause* cl, Clause* premise, Clause* replacement=0);
  virtual void onClauseReduction(Clause* cl, ClauseIterator premises, Clause* replacement=0) {}
  virtual void onNewClause(Clause* cl) {}
  virtual void onAllProcessed() {}

  const Options& getOptions() const;
protected:
  class CompRec
  {
    LiteralStack _lits;
    bool _isSpecial;
  public:
    CompRec() {}
    CompRec(Literal** lits, unsigned len);

    void add(Literal* l) { _lits.push(l); }
    void markSpecial() { _isSpecial = true; }

    unsigned size() const { return _lits.size(); }
    Literal* operator[](unsigned i) const { return _lits[i]; }
    Literal* const * array() const { return _lits.begin(); }

   
    bool special() const { return _isSpecial; }

    class Iterator : public LiteralStack::ConstIterator {
    public:
      Iterator(const CompRec& obj) : LiteralStack::ConstIterator(obj._lits) {}
    };
  };

  bool isSpecial(Literal* l);
  virtual bool canStandAlone(Literal* lit);
  virtual bool standAloneObligations();

  bool getComponents(Clause* cl, Stack<CompRec>& acc, bool putSpecialsTogether);


  bool splitPositive();

  virtual bool splittingAllowed(Clause* cl);

  bool isAnswerLiteral(Literal* lit);

  SaturationAlgorithm* _sa;
  AnswerLiteralManager* _ansLitMgr;
};

};

#endif
