
/*
 * File AIGDefinitionIntroducer.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __AIGDefinitionIntroducer__
#define __AIGDefinitionIntroducer__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"
#include "Lib/DHSet.hpp"
#include "Lib/Stack.hpp"

#include "AIG.hpp"
#include "AIGRewriter.hpp"


namespace Shell {

class AIGDefinitionIntroducer : public ScanAndApplyFormulaUnitTransformer
{
  typedef SharedSet<unsigned> VarSet;

  struct NodeInfo {

    

   
    bool _hasQuant[2];

    bool _hasName;
    AIGRef _name;

    VarSet* _freeVars;
   
    Color _clr;

   
    unsigned _directRefCnt;

    

   
    bool _inPol[2];
   
    bool _inQuant[2];

   
    unsigned _formRefCnt;


    

   
    FormulaUnit* _namingUnit;
  };

  
  bool _eprPreserving;
  unsigned _namingRefCntThreshold;
  unsigned _mergeEquivDefs;

  mutable AIGFormulaSharer _fsh;
  AIGRewriter _arwr;

  typedef Stack<AIGRef> TopLevelStack;
  TopLevelStack _toplevelAIGs;

 
  AIGStack _refAIGs;
 
  Stack<NodeInfo> _refAIGInfos;
 
  DHMap<AIGRef,size_t> _aigIndexes;

 
  DHMap<AIGRef,AIGRef> _existingDefs;
  DHMap<AIGRef,FormulaUnit*> _existingDefUnits;

 
  AIGRewriter::RefMap _defsSaturated;

 
  Stack<FormulaUnit*> _newDefs;



  
 
  DHSet<unsigned> _introducedPreds;
 
  DHMap<Literal*,AIGRef> _introducedAtoms;


  AIGRef getPreNamingAig(unsigned aigStackIdx);

  bool shouldIntroduceName(unsigned aigStackIdx);
  Literal* getNameLiteral(unsigned aigStackIdx);
  void introduceName(unsigned aigStackIdx);

  bool scanDefinition(FormulaUnit* def);

  
  void collectTopLevelAIGsAndDefs(UnitList* units);
  void processTopLevelAIGs();

  
  void doFirstRefAIGPass();
  void doSecondRefAIGPass();
  void doThirdRefAIGPass();

  VarSet* getAtomVars(Literal* l);
  NodeInfo& getNodeInfo(AIGRef r);

  FormulaUnit* createNameUnit(AIGRef rhs, AIGRef atomName);

  Inference* getInferenceFromPremIndexes(Unit* orig, AIGRewriter::PremiseSet* premIndexes);

public:
  AIGDefinitionIntroducer(unsigned threshold=4);

  virtual void scan(UnitList* units);

  using ScanAndApplyFormulaUnitTransformer::apply;
  virtual bool apply(FormulaUnit* unit, Unit*& res);

  virtual UnitList* getIntroducedFormulas();

 
  const DHSet<unsigned>& introducedPreds() const { return _introducedPreds; }
 
  Formula* getNamedFormula(Literal* nameAtom, Unit*& premise) const;
protected:
  virtual void updateModifiedProblem(Problem& prb);
};

}

#endif 
