
/*
 * File AIGRewriter.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __AIGRewriter__
#define __AIGRewriter__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"

#include "AIG.hpp"

namespace Shell {

using namespace Lib;

class AIGRewriter {
  struct L0DerefQueryRec;

  AIG& _aig;

  DHMap<unsigned, Unit*> _premMap;
public:
  unsigned getPremiseNum(Unit* u);
  Unit* getPremiseUnit(unsigned u);

  typedef SharedSet<unsigned> PremiseSet;
  typedef AIGRef Ref;

  typedef Stack<PremiseSet*> PremSetStack;

  typedef pair<Ref,PremiseSet*> PremRef;

 
  typedef DHMap<Ref,PremRef> RefMap;

  static Ref lev0Deref(Ref r, RefMap& map, PremiseSet** prem);
  Ref lev1Deref(Ref r, RefMap& map, PremiseSet** premAcc);
private:

  typedef MapToLIFO<Ref,Ref> RefEdgeMap;



  void collectUsed(Ref r, const RefMap& map, RefEdgeMap& edges);

  void saturateOnTopSortedStack(const AIGStack& stack, RefMap& map);

public:
  AIGRewriter(AIG& aig) : _aig(aig) {}


  void restrictToGetOrderedDomain(RefMap& map, AIGStack& domainOrder);
  void saturateMap(RefMap& map, Stack<Ref>* finalDomain=0);

  void applyWithCaching(Ref r, RefMap& map);
  void makeIdempotent(RefMap& map, Stack<Ref>* finalDomain=0);

};

}

#endif 
