
/*
 * File SpawningCM.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include <stdlib.h>
#include <csignal>

#include "Lib/Environment.hpp"
#include "Lib/Int.hpp"
#include "Lib/List.hpp"
#include "Lib/System.hpp"

#include "Kernel/Problem.hpp"
#include "Kernel/Unit.hpp"

#include "Shell/Options.hpp"
#include "Shell/UIHelper.hpp"

#include "SpawningCM.hpp"

namespace Shell
{
namespace CASC
{

SpawningCM::SpawningCM(string executable)
: _executable(executable)
{
  CALL("SpawningCM::SpawningCM");

  if(!system(0)) {
    USER_ERROR("The spawning CASC mode is not supported on this system (the \"int system(const char *)\" function is not available)");
  }

  if(env.options->inputFile()=="") {
    USER_ERROR("Value of the --input_file option must be specified for the CASC mode in Windows.");
  }
  _inputFile=env.options->inputFile();

  
  ScopedPtr<Problem> prb(UIHelper::getInputProblem(*env.options));
  _property = Property::scan(prb->units());
}

bool SpawningCM::runSlice(Options& opt)
{
  CALL("SpawningCM::runSlice");

  string strategy=opt.generateTestId();

  string cmdLine=_executable+" -decode "+strategy+" -input_file "+_inputFile;

  if(env.options->include()!="") {
    cmdLine+=" -include "+env.options->include();
  }

#if COMPILER_MSVC
  
  _flushall();

  
  System::ignoreSIGINT();
#endif

  int res=system(cmdLine.c_str());

#if COMPILER_MSVC
  
  System::heedSIGINT();

  if(res==3)  {
    
    
    
    handleSIGINT();
  }

  if(res==0) {
    return true;
  }
#else
  if( (WIFSIGNALED(res) && WTERMSIG(res)==SIGINT) ||
      (WIFEXITED(res) && WEXITSTATUS(res)==3) )  {
    
    
    
    handleSIGINT();
  }

  if(WIFEXITED(res) && WEXITSTATUS(res)==0) {
    
    return true;
  }
#endif

  return false;
}

}
}
