
/*
 * File EPRInlining.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __EPRInlining__
#define __EPRInlining__

#include "Forwards.hpp"

#include "Lib/Array.hpp"
#include "Lib/DArray.hpp"
#include "Lib/Deque.hpp"
#include "Lib/DHMap.hpp"
#include "Lib/DHSet.hpp"
#include "Lib/MapToLIFO.hpp"
#include "Lib/Set.hpp"
#include "Lib/Stack.hpp"

#include "PDInliner.hpp"


namespace Shell {

class EPRRestoring {
public:
  void scan(UnitList* units);
  virtual ~EPRRestoring() {}
protected:
  EPRRestoring(bool trace) : _trace(trace) {}

  virtual void processActiveDefinitions(UnitList* units) {}


 
  DHMap<Unit*, unsigned> _defPreds;

 
  Stack<unsigned> _activePreds;

 
  DHSet<Unit*> _activeUnits;

  ZIArray<FormulaUnit*> _nonEprDefs;
 
  ZIArray<bool> _nonEprReversedPolarity;
 
  ZIArray<int> _nonEprDefPolarities;

  bool _trace;
private:
  bool scanDefinition(FormulaUnit* unit);
  void performClosure();

  static bool isNonEPRDef(Literal* lhs, Formula* body, int& polarity);
  static int combinePolarities(int p1, int p2);

  bool addNEDef(FormulaUnit* unit, unsigned functor, int polarity);

  Deque<unsigned> _newNEPreds;

  Stack<unsigned> _nonEprPreds;

  typedef MapToLIFO<unsigned,pair<FormulaUnit*,int> > DepMap;

 
  DepMap _dependent;
};

class EPRInlining : public EPRRestoring {
public:
  EPRInlining(bool trace=false) : EPRRestoring(trace), _inliner(false, trace) {}

  Unit* apply(Unit* unit);
  bool apply(UnitList*& units);
  void apply(Problem& prb);
protected:
  virtual void processActiveDefinitions(UnitList* units);
private:
  PDInliner _inliner;
};

}

#endif 
