
/*
 * File EPRSkolem.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __EPRSkolem__
#define __EPRSkolem__

#include "Forwards.hpp"

#include "EPRInlining.hpp"

namespace Shell {

class EPRSkolem : public EPRRestoring {
public:
  EPRSkolem(bool trace=false) : EPRRestoring(trace), _defs(0) {}

  void apply(Problem& prb);
  bool apply(UnitList*& units);
  bool apply(Unit* unit, UnitList*& res);

  static FormulaUnit* constantSkolemize(FormulaUnit* unit);
protected:
  virtual void processActiveDefinitions(UnitList* units);
private:
  void enableLiteralHeader(unsigned header);
  void processLiteralHeader(Literal* lit, unsigned header);
  void processProblemLiteral(Literal* lit, int polarity);
  void processProblemClause(Clause* cl);
  void processProblemFormula(FormulaUnit* fu);

  bool applyToDefinitionHalf(FormulaUnit* fu, Literal* lhs, Formula* rhs,
      int topPolarity, UnitList*& res);
  void processDefinition(FormulaUnit* unit);

  FormulaUnit* definitionToImplication(FormulaUnit* premise, Literal* lhs,
      Formula* rhs, int topPolarity);

  static string headerToString(unsigned hdr);

  class Applicator;
  class ConstantSkolemizer;
  class CannotEPRSkolemize : public Exception {};


  typedef MapToLIFO<unsigned,Literal*> InstMap;

 
  InstMap _insts;

  DHSet<unsigned> _inlinedHeaders;

 
  DHSet<unsigned> _haveNonground;

  typedef DHMap<Unit*,UnitList*> ReplacementMap;

 
  ReplacementMap _replacements;

  UnitList* _defs;
};

}

#endif 
