
/*
 * File EqResWithDeletion.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#include "Kernel/Clause.hpp"
#include "Kernel/Inference.hpp"
#include "Kernel/Problem.hpp"
#include "Kernel/SubstHelper.hpp"
#include "Kernel/Term.hpp"
#include "Kernel/Unit.hpp"

#include "EqResWithDeletion.hpp"

namespace Shell
{

using namespace Lib;
using namespace Kernel;

void EqResWithDeletion::apply(Problem& prb)
{
  CALL("EqResWithDeletion::apply");

  if(apply(prb.units())) {
    prb.invalidateByRemoval();
  }
}

bool EqResWithDeletion::apply(UnitList*& units)
{
  CALL("EqResWithDeletion::apply(UnitList*&)");

  bool modified = false;

  UnitList::DelIterator uit(units);
  while(uit.hasNext()) {
    Clause* cl=static_cast<Clause*>(uit.next());
    ASS(cl->isClause());
    Clause* cl2=apply(cl);
    if(cl!=cl2) {
      modified = true;
      uit.replace(cl2);
    }
  }
  return modified;
}

Clause* EqResWithDeletion::apply(Clause* cl)
{
  CALL("EqResWithDeletion::apply(Clause*)");

  
start_applying:

  unsigned clen=cl->length();

  _subst.reset();

  static Stack<Literal*> resLits(8);
  resLits.reset();

  bool foundResolvable=false;
  for(unsigned i=0;i<clen;i++) {
    Literal* lit=(*cl)[i];
    if(!foundResolvable && scan(lit)) {
      foundResolvable=true;
    } else {
      resLits.push(lit);
    }
  }
  if(!foundResolvable) {
    return cl;
  }

  unsigned nlen=resLits.size();
  ASS_L(nlen, clen);

  Inference* inf = new Inference1(Inference::EQUALITY_RESOLUTION, cl);
  Clause* res = new(nlen) Clause(nlen, cl->inputType(), inf);
  res->setAge(cl->age());

  for(unsigned i=0;i<nlen;i++) {
    (*res)[i] = SubstHelper::apply(resLits[i], *this);
  }

  cl=res;
  goto start_applying;
}

TermList EqResWithDeletion::apply(unsigned var)
{
  TermList res;
  if(_subst.find(var, res)) {
    return res;
  } else {
    return TermList(var, false);
  }
}

bool EqResWithDeletion::scan(Literal* lit)
{
  CALL("EqResWithDeletion::scan(Literal*)");

  if(lit->isEquality() && lit->isNegative()) {
    TermList t0=*lit->nthArgument(0);
    TermList t1=*lit->nthArgument(1);
    if( t0.isVar() && !t1.containsSubterm(t0) ) {
      if(_subst.insert(t0.var(), t1)) {
	return true;
      }
    }
    if( t1.isVar() && !t0.containsSubterm(t1) ) {
      if(_subst.insert(t1.var(), t0)) {
	return true;
      }
    }
  }
  return false;

}

}
