
/*
 * File EqualityAxiomatizer.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __EqualityAxiomatizer__
#define __EqualityAxiomatizer__

#include "Forwards.hpp"

#include "Lib/DHSet.hpp"

#include "Options.hpp"

namespace Shell {

using namespace Lib;
using namespace Kernel;

class EqualityAxiomatizer {
private:
  Options::EqualityProxy _opt;

  typedef DHSet<unsigned> SymbolSet;
  typedef DHSet<unsigned> SortSet;

  SymbolSet _fns;
  SymbolSet _preds;
  SortSet _eqSorts;

  void saturateEqSorts();

  void addLocalAxioms(UnitList*& units, unsigned sort);
  void addCongruenceAxioms(UnitList*& units);
  Clause* getFnCongruenceAxiom(unsigned fn);
  Clause* getPredCongruenceAxiom(unsigned pred);
  bool getArgumentEqualityLiterals(BaseType* symbolType, LiteralStack& lits, Stack<TermList>& vars1,
      Stack<TermList>& vars2);

  void scan(UnitList* units);
  void scan(Literal* lit);
  UnitList* getAxioms();
public:
 
  EqualityAxiomatizer(Options::EqualityProxy opt) : _opt(opt)
  {
    ASS_REP(opt==Options::EP_RS || opt==Options::EP_RST || opt==Options::EP_RSTC, opt);
  }

  void apply(Problem& prb);
  void apply(UnitList*& units);

};

}

#endif 
