
/*
 * File EqualityPropagator.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __EqualityPropagator__
#define __EqualityPropagator__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"
#include "Lib/Stack.hpp"

#include "Kernel/Formula.hpp"
#include "Kernel/Term.hpp"

namespace Shell {

using namespace Lib;
using namespace Kernel;

class EqualityPropagator {
public:
  EqualityPropagator(bool trace=false) : _trace(trace) {}

  void apply(Problem& prb);
  bool apply(UnitList*& units);
  Unit* apply(Unit* u);
  TermList apply(unsigned var);
private:

  class SingletonVariableRemover;

  FormulaUnit* apply(FormulaUnit* u);
  Formula* apply(Formula* form);
  Formula* applyTopLevel(Formula* form);

  Formula* apply(JunctionFormula* form);
  Formula* apply(QuantifiedFormula* form);
  Literal* apply(Literal* l);

  int getVarPolarity(unsigned var);

  void collectResolvableBindings(Formula* form, bool negated);

  void undoBindings(unsigned remaining);

  bool _trace;

  Stack<unsigned> _vars;
  DHMap<unsigned,unsigned> _varDepths;

  Stack<unsigned> _boundVars;
  DHMap<unsigned,TermList> _varBindings;
};

}

#endif 
