
/*
 * File FunctionDefinition.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __FunctionDefinition__
#define __FunctionDefinition__

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"
#include "Lib/MultiCounter.hpp"
#include "Lib/Stack.hpp"
#include "Kernel/Unit.hpp"

namespace Shell {

using namespace Lib;
using namespace Kernel;

class FunctionDefinition
{
public:
  struct Def;
  FunctionDefinition();
  ~FunctionDefinition();

  void removeAllDefinitions(Problem& prb);
  bool removeAllDefinitions(UnitList*& units);

  static void removeUnusedDefinitions(Problem& prb);
  static bool removeUnusedDefinitions(UnitList*& units, Problem* prb=0);


  static Def* isFunctionDefinition (Unit&);
  static void deleteDef(Def* def);






private:
  static Def* isFunctionDefinition (Clause*);
  static Def* isFunctionDefinition (FormulaUnit&);
  static Def* isFunctionDefinition (Literal*);
  static Def* defines (Term* lhs, Term* rhs);
  static bool occurs (unsigned function, Term&);


  bool isDefined(Term* t);

  Term* applyDefinitions(Literal* t, Stack<Def*>* usedDefs);
  Clause* applyDefinitions(Clause* cl);

  void checkDefinitions(Def* t);
  void assignArgOccursData(Def* d);












  typedef DHMap<int, Def*, IdentityHash> Fn2DefMap;
  Fn2DefMap _defs;

 
  Stack<Def*> _blockedDefs;

  Stack<Def*> _safeDefs;
 
  MultiCounter _counter;
 
  int _found;
 
  int _removed;

  Problem* _processedProblem;
}; 


}

#endif
