
/*
 * File Grounding.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __Grounding__
#define __Grounding__

#include "Forwards.hpp"

#include "Lib/DArray.hpp"
#include "Lib/DHMap.hpp"
#include "Lib/Stack.hpp"
#include "Lib/VirtualIterator.hpp"

namespace Shell
{

using namespace Kernel;
using namespace Lib;

class Grounding
{
public:
  static ClauseList* simplyGround(ClauseIterator clauses);

  ClauseList* ground(Clause* clause);


  static ClauseList* getEqualityAxioms(bool otherThanReflexivity);
private:
  static void getLocalEqualityAxioms(unsigned sort, bool otherThanReflexivity, ClauseList*& acc);


  struct GroundingApplicator
  {
    GroundingApplicator();
    void initForClause(Clause* cl);
    bool newAssignment();
    TermList apply(unsigned var);
  private:
    DHMap<unsigned, unsigned, IdentityHash> _varNumbering;
    Stack<TermList> _constants;
    DArray<unsigned> _indexes;
    unsigned _maxIndex;
    int _varCnt;
    bool _beforeFirst;
  };


  GroundingApplicator _ga;
};

}

#endif
