
/*
 * File LaTeX.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __LaTeX__
#define __LaTeX__

#include <string>

#include "Forwards.hpp"

#include "Lib/DHMap.hpp"
#include "Lib/Stack.hpp"

#include "Kernel/Connective.hpp"
#include "Kernel/InferenceStore.hpp"











namespace Shell {

using namespace Kernel;

class LaTeX
{
public:
  LaTeX() : _nextNodeNum(0) {}

  string refutationToString(Unit* ref);



  string toString(const Term&) const;
  string toString(const string& funOrPred,const TermList& args) const;
  string toString(Unit*);
  string toString(Unit*,BDDNode*);
private:




  string varToString(unsigned num) const;
  string toString(TermList*) const;
  string toString(Literal*) const;
  string toString(Clause*);
  string toString(Clause*, BDDNode*);
  string toString(Formula*) const;
  string toString(Formula*, Connective c) const;

  string getClauseLatexId(UnitSpec cs);

  string splittingToString(InferenceStore::SplittingRecord*);
  string toStringAsInference(Unit*);
  string toStringAsInference(UnitSpec cs, InferenceStore::FullInference* inf);

  string symbolToString (unsigned num, bool pred) const;


  string toString(BDDNode*);
  string getBDDVarName(int var);

  Stack<string> definitionStack;
  int _nextNodeNum;
  DHMap<BDDNode*,string> _nodeNames;
}; 


}

#endif 

