
/*
 * File Normalisation.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __Normalisation__
#define __Normalisation__

#include "Forwards.hpp"

#include "Lib/Comparison.hpp"
#include "Kernel/Unit.hpp"

#include "SymCounter.hpp"

namespace Shell {

using namespace Lib;
using namespace Kernel;

class Normalisation
{
public:
  Normalisation();
  void normalise(Problem&);
  UnitList* normalise(UnitList*);
  bool lessThan(Literal*, Literal*);
  bool lessThan(Unit*, Unit*);
private:
  void normalise(Unit*);
  Comparison compare(Term*, Term*);
  Comparison compare(Formula*, Formula*);
  Comparison compare(Literal*, Literal*);
  bool lessThan(Formula*, Formula*);
  bool lessThan(Clause*, Clause*);
  Comparison compare(TermList* ss, TermList* ts);

 
  inline static
  Comparison compare (int i1, int i2)
  {
    return i1 > i2
           ? GREATER
           : i1 == i2
             ? EQUAL
             : LESS;
  }

 
  inline static
  Comparison compare (bool b1, bool b2)
  {
    return b1 ? (b2 ? EQUAL : LESS) : (b2 ? GREATER : EQUAL);
  }

 
  SymCounter _counter;
}; 

}

#endif
