
/*
 * File PDMerger.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __PDMerger__
#define __PDMerger__

#include "Forwards.hpp"

#include "Lib/Comparison.hpp"
#include "Lib/Deque.hpp"
#include "Lib/DHMap.hpp"
#include "Lib/ScopedPtr.hpp"
#include "Lib/Stack.hpp"

#include "Indexing/FormulaIndex.hpp"

#include "PDInliner.hpp"

namespace Shell {

using namespace Lib;
using namespace Kernel;
using namespace Indexing;

class PDMerger {
public:
  PDMerger(bool trace=false);
  ~PDMerger();

  void scan(UnitList* units);
  bool apply(UnitList*& units);
  void apply(Problem& prb);
  Unit* apply(Unit* unit);
  FormulaUnit* apply(FormulaUnit* unit);

private:
  void processDefinition(FormulaUnit* unit);

  void handleIndexes(FormulaUnit* unit, bool insert);
  void triggerNewDefinitions(unsigned elPred);

  class Normalizer;

  struct UnitComparator
  {
    static Comparison compare(Unit* u1, Unit* u2);
  };

  bool _trace;

  ScopedPtr<FormulaIndex> _index;

  DHMap<FormulaUnit*,FormulaUnit*> _replacements;
  PDInliner _inliner;

  typedef Deque<FormulaUnit*> FormulaQueue;

 
  FormulaQueue _unprocessed;

  typedef SkipList<FormulaUnit*, UnitComparator> FormulaSkipList;

 
  DArray<FormulaSkipList*> _pred2Defs;
};

}

#endif 
