
/*
 * File PredicateDefinition.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __PredicateDefinition__
#define __PredicateDefinition__

#include "Forwards.hpp"

#include "Lib/DArray.hpp"
#include "Lib/DHMap.hpp"
#include "Lib/VirtualIterator.hpp"
#include "Kernel/FormulaUnit.hpp"

#include "SymCounter.hpp"

namespace Shell {

using namespace Lib;
using namespace Kernel;

class PredicateDefinition
{
public:
  typedef DHMap<Unit*, Unit*> ReplMap;

  PredicateDefinition(bool trace=false);
  ~PredicateDefinition();

  void collectReplacements(UnitList* units, ReplMap& unitReplacements);

  void removeUnusedDefinitionsAndPurePredicates(Problem& prb);
  void removeUnusedDefinitionsAndPurePredicates(UnitList*& units);

  void addBuiltInPredicate(unsigned pred);
private:
  struct Def;
  struct PredData;

  void scan(Unit* u);
  void scan(Clause* u);
  void scan(FormulaUnit* u);
  void count (Formula* f,int polarity,int add, Unit* unit);
  void count (FormulaUnit* f,int add) { count(f->formula(), 1, add, f); }
  void count (Clause* cl,int add);
  void count (Unit* u,int add);
  bool tryGetDef(Literal* lhs, Formula* rhs, FormulaUnit* unit);

  FormulaUnit* replacePurePredicates(FormulaUnit* u);
  Formula* replacePurePredicates(Formula* u);
  Clause* replacePurePredicates(Clause* u);
  Unit* replacePurePredicates(Unit* u);

  FormulaUnit* makeImplFromDef(FormulaUnit* def, unsigned pred, bool forward);

  Unit* getReplacement(Unit* u, ReplMap& replacements);
  FormulaUnit* getReplacement(FormulaUnit* u, ReplMap& replacements);

  void eliminatePredicateDefinition(unsigned pred, ReplMap& replacements);
  void replacePurePred(unsigned pred, ReplMap& replacements);

  bool _trace;

  Problem* _processedPrb;

  unsigned _predCnt;
  PredData* _preds;

  DHMap<unsigned, Def*> _defs;
  DHMap<unsigned, bool> _purePreds;
  Stack<int> _eliminable;
  Stack<int> _pureToReplace;
};

};

#endif
