
/*
 * File Profile.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __Profile__
#define __Profile__

#include "Kernel/Unit.hpp"
#include "Lib/Array.hpp"

namespace Kernel
{
  class Clause;
  class Literal;
  class Term;
  class TermList;
}

using namespace std;
using namespace Kernel;
using namespace Lib;

#define HOW_MANY 10

namespace Shell {

class Profile 
{
 public:
  
  Profile();
  ~Profile();

  void scan(const UnitList*);
  string toString() const;
  bool lessThan(const Clause*,const Clause*);
  bool lessThan(const Literal*,const Literal*);

 private:
  void scan(const Clause*);
  void scan(const Literal*);
  void rescan(const Literal*);
  void descan(const Literal*);
  void scan(const TermList*);
  void scan(const Term*);
  int evaluate(const Clause* c);

  void output(const Clause* clause,ostream& str);
  void output(const Literal* clause,ostream& str,char pred);
  void output(const TermList* clause,ostream& str);
  void output(const Term* clause,ostream& str);


 
  unsigned _numberOfClauses;
 
  unsigned _varsInThisClause;
 
  unsigned _symsInThisClause;
 
  const Clause* _currentClause;
 
  int _vars[HOW_MANY];
 
  int _syms[HOW_MANY];
 
  int _lits[HOW_MANY];
 
  int* _funs;
 
  int* _posPreds;
 
  int* _negPreds;
 
  const Clause** _funClauses;
 
  const Clause** _predClauses;

 
  class VarCounter
    : public Array<const Clause*>
  {
  public:
    VarCounter();
    void fillInterval(size_t start,size_t end);
  }; 

  VarCounter vc;
}; 

}

#endif 
