
/*
 * File Rectify.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __Rectify__
#define __Rectify__

#include "Lib/Array.hpp"
#include "Lib/List.hpp"

#include "Kernel/Formula.hpp"
#include "Kernel/SubstHelper.hpp"

namespace Kernel {
  class Unit;
  class Clause;
  class Term;
}

using namespace Kernel;

namespace Shell {

class Rectify 
{
public:
 
  Rectify()
    : _free(0), _removeUnusedVars(true)
  {}
  static FormulaUnit* rectify(FormulaUnit*, bool removeUnusedVars=true);
  static void rectify(UnitList*& units);
private:
  typedef List<int> VarList;
  typedef pair<int,bool> VarWithUsageInfo;
  typedef List<VarWithUsageInfo> VarUsageTrackingList;
 
  class Renaming
    : public Array<VarUsageTrackingList*>
  {
  public:
    Renaming()
      : Array<VarUsageTrackingList*>(15),
	_nextVar(0), _used(0)
    {
      fillInterval(0,15);
    }
    ~Renaming();
    bool tryGetBoundAndMarkUsed (int var,int& boundTo) const;
    VarWithUsageInfo getBoundAndUsage(int var) const;
    int bind (int v);
    void undoBinding(int v);
  private:
    virtual void fillInterval (size_t start,size_t end);
   
    int _nextVar;
   
    DHSet<unsigned>* _used;
  };

  void reset();

  unsigned rectifyVar(unsigned v);
  void rectifyTermLet(TermList& origin, TermList& target);
  void rectifyFormulaLet(Literal*& origin, Formula*& target);

  Formula* rectify(Formula*);
  FormulaList* rectify(FormulaList*);
  void bindVars(VarList*);
  void unbindVars(VarList*);
  VarList* rectifyBoundVars(VarList*);
  TermList rectify(TermList);
  Term* rectify(Term* t);
  Term* rectifySpecialTerm(Term* t);
  Literal* rectify(Literal*);
  Literal* rectifyShared(Literal* lit);
  bool rectify(TermList* from,TermList* to);

  friend class Kernel::SubstHelper;
 
  TermList apply(unsigned var) { return TermList(rectifyVar(var), false); }

 
  Renaming _renaming;
 
  List<int>* _free;

 
  bool _removeUnusedVars;





}; 

}

#endif
