
/*
 * File SineUtils.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */



#ifndef __SineUtils__
#define __SineUtils__

#include "Forwards.hpp"

#include "Lib/DArray.hpp"
#include "Lib/Stack.hpp"

namespace Shell {

using namespace Lib;
using namespace Kernel;

class SineSymbolExtractor
{
public:
  typedef unsigned SymId;
  typedef VirtualIterator<SymId> SymIdIterator;

  SymId getSymIdBound();

  SymIdIterator extractSymIds(Unit* u);

  void decodeSymId(SymId s, bool& pred, unsigned& functor);
  bool validSymId(SymId s);
private:
  void addSymIds(Literal* lit, int polarity, Stack<SymId>& ids) const;

  void extractFormulaSymbols(Formula* f,int polarity,Stack<SymId>& itms);
};


class SineBase
{
protected:
  typedef SineSymbolExtractor::SymId SymId;
  typedef SineSymbolExtractor::SymIdIterator SymIdIterator;

  void initGeneralityFunction(UnitList* units);

 
  DArray<unsigned> _gen;

  SineSymbolExtractor _symExtr;
};

class SineSelector
: public SineBase
{
public:
  SineSelector(const Options& opt);
  SineSelector(bool onIncluded, float tolerance, unsigned depthLimit, unsigned genThreshold=0);

  void perform(UnitList*& units);
  void perform(Problem& prb);
private:
  void init();

  void updateDefRelation(Unit* u);

  bool _onIncluded;
  bool _strict;
  unsigned _genThreshold;
  float _tolerance;
  unsigned _depthLimit;

 
  DArray<UnitList*> _def;

 
  Stack<Unit*> _unitsWithoutSymbols;
};


class SineTheorySelector
: public SineBase
{
public:
  SineTheorySelector(const Options& opt);

  void initSelectionStructure(UnitList* units);
  void perform(UnitList*& units);
private:

 
  static const unsigned short maxTolerance=50;
  static const unsigned short strictTolerance=10;

  void handlePossibleSignatureChange();

  void updateDefRelation(Unit* u);

  unsigned _genThreshold;

  struct DEntry
  {
    DEntry(unsigned short minTolerance, Unit* unit) : minTolerance(minTolerance), unit(unit) {}

    unsigned short minTolerance;
    Unit* unit;
  };
  typedef List<DEntry> DEntryList;

 
  DArray<DEntryList*> _def;

 
  Stack<Unit*> _unitsWithoutSymbols;

  const Options& _opt;
};


}

#endif
