
/*
 * File SpecialTermElimination.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __SpecialTermElimination__
#define __SpecialTermElimination__

#include "Forwards.hpp"


namespace Shell {

using namespace Kernel;

class SpecialTermElimination {
public:
  SpecialTermElimination();

  void apply(Problem& prb);
  void apply(UnitList*& units);
  FormulaUnit* apply(FormulaUnit* fu, UnitList*& defs);
  static bool hasSpecials(FormulaUnit* fu);
private:
  class LetSpec
  {
    bool _term;
    union {
      struct {
	size_t lhs; 
	size_t rhs;
      } _t;
      struct {
	Literal* lhs;
	Formula* rhs;
      } _f;
    };
  public:
    LetSpec(TermList o, TermList t) : _term(true)
    {
      CALL("SpecialTermElimination::LetSpec::LetSpec(TermList...)");
      ASS(o.isSafe());
      
      
      ASS(!o.isTerm() || o.term()->weight()==o.term()->arity()+1);
      ASS(!o.isTerm() || o.term()->getDistinctVars()==o.term()->arity());

      _t.lhs = o.content();
      _t.rhs = t.content();
    }
    LetSpec(Literal* o, Formula* t) : _term(false)
    {
      CALL("SpecialTermElimination::LetSpec::LetSpec(Literal*...)");
      
      
      ASS_EQ(o->weight(),o->arity()+1);
      ASS_EQ(o->getDistinctVars(),o->arity());


      _f.lhs = o;
      _f.rhs = t;
    }
    bool term() const { return _term; }
    TermList tLhs() const { ASS(term()); return TermList(_t.lhs); }
    TermList tRhs() const { ASS(term()); return TermList(_t.rhs); }
    Literal* fLhs() const { ASS(!term()); return _f.lhs; }
    Formula* fRhs() const { ASS(!term()); return _f.rhs; }
  };

  TermList processSpecialTerm(Term* t);

  TermList process(TermList t);
  Term* process(Term* t);
  Formula* process(Formula* f);
  FormulaList* process(FormulaList* fs);
  FormulaUnit* apply(FormulaUnit* fu);

  Term* eliminateTermIte(Formula * condition, TermList thenBranch, TermList elseBranch);

  bool checkForTermLetReplacement(TermList t, TermList& res);

  bool eliminatingTermLet() const { return _letStack.isNonEmpty() && _letStack.top().term(); }
  bool eliminatingFormulaLet() const { return _letStack.isNonEmpty() && !_letStack.top().term(); }
  bool eliminatingTermIte() const { return _letStack.isEmpty(); }

  UnitList* _defs;

 
  DHMap<unsigned,unsigned> _currentFormulaVarSorts;

 
  Stack<LetSpec> _letStack;

  Problem* _currentPrb;
};

}

#endif 
