
/*
 * File Statistics.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __Statistics__
#define __Statistics__

#include <ostream>

extern const char* VERSION_STRING;

namespace Kernel {
  class Unit;
}

namespace Shell {

using namespace std;

class Statistics
{
public:
  Statistics();

  void print(ostream& out);

  
 
  unsigned inputClauses;
 
  unsigned inputFormulas;

  
 
  unsigned formulaNames;
 
  unsigned initialClauses;
 
  unsigned splittedInequalities;
 
  unsigned propagatedEqualities;
 
  unsigned removedSingletonVariables;
 
  unsigned purePredicates;
 
  unsigned trivialPredicates;
 
  unsigned hornReversedPredicates;
 
  unsigned eprPreservingSkolemizations;
 
  unsigned inlinedPredicateDefinitions;
 
  unsigned mergedPredicateDefinitions;
 
  unsigned unusedPredicateDefinitions;
 
  unsigned functionDefinitions;
 
  unsigned selectedBySine;
 
  unsigned sineIterations;

  
 
  unsigned factoring;
 
  unsigned resolution;
 
  unsigned urResolution;
 
  unsigned forwardSuperposition;
 
  unsigned backwardSuperposition;
 
  unsigned selfSuperposition;
 
  unsigned equalityFactoring;
 
  unsigned equalityResolution;

  
 
  unsigned duplicateLiterals;
 
  unsigned trivialInequalities;
 
  unsigned forwardSubsumptionResolution;
 
  unsigned backwardSubsumptionResolution;
 
  unsigned forwardDemodulations;
 
  unsigned forwardDemodulationsToEqTaut;
 
  unsigned backwardDemodulations;
 
  unsigned backwardDemodulationsToEqTaut;
 
  unsigned forwardLiteralRewrites;
 
  unsigned condensations;
 
  unsigned globalSubsumption;
 
  unsigned evaluations;
 
  unsigned interpretedSimplifications;

  
 
  unsigned simpleTautologies;
 
  unsigned equationalTautologies;
 
  unsigned forwardSubsumed;
 
  unsigned backwardSubsumed;
 
  unsigned subsumedEmptyClauses;
 
  unsigned emptyClauseSubsumptions;
 
  unsigned subsumedByMarking;

  
 
  unsigned generatedClauses;
 
  unsigned passiveClauses;
 
  unsigned activeClauses;

  unsigned discardedNonRedundantClauses;

  unsigned inferencesSkippedDueToColors;

 
  unsigned finalPassiveClauses;
 
  unsigned finalActiveClauses;

 
  unsigned reactivatedClauses;

  unsigned splitClauses;
  unsigned splitComponents;
  unsigned uniqueComponents;
 
  unsigned splittingNamesIntroduced;
 
  unsigned bddPropClauses;
 
  unsigned satClauses;
 
  unsigned unitSatClauses;
 
  unsigned binarySatClauses;
 
  unsigned learntSatClauses;
 
  unsigned learntSatLiterals;
 
  size_t bddMemoryUsage;

  unsigned backtrackingSplits;
  unsigned backtrackingSplitsRefuted;
  unsigned backtrackingSplitsRefutedZeroLevel;

  unsigned satSplits;
  unsigned satSplitRefutations;

  unsigned instGenGeneratedClauses;
  unsigned instGenRedundantClauses;
  unsigned instGenKeptClauses;
  unsigned instGenIterations;

  unsigned maxBFNTModelSize;

 
  unsigned satPureVarsEliminated;

 
  enum TerminationReason {
   
    REFUTATION,
   
    SATISFIABLE,
   
    REFUTATION_NOT_FOUND,
   
    UNKNOWN,
   
    TIME_LIMIT,
   
    MEMORY_LIMIT
  };
 
  TerminationReason terminationReason;
 
  Kernel::Unit* refutation;
 
  Kernel::UnitList* saturatedSet;
 
  string model;

  enum ExecutionPhase {
   
    INITIALIZATION,
    PARSING,
   
    PROPERTY_SCANNING,
    NORMALIZATION,
    SINE_SELECTION,
    INCLUDING_THEORY_AXIOMS,
    PREPROCESS_1,
    EQUALITY_PROPAGATION,
    PREDIACTE_DEFINITION_MERGING,
    EPR_PRESERVING_SKOLEMIZATION,
    PREDICATE_DEFINITION_INLINING,
    UNUSED_PREDICATE_DEFINITION_REMOVAL,
    PREPROCESS_2,
    NAMING,
    PREPROCESS_3,
    CLAUSIFICATION,
    FUNCTION_DEFINITION_ELIMINATION,
    INEQUALITY_SPLITTING,
    EQUALITY_RESOLUTION_WITH_DELETION,
    EQUALITY_PROXY,
    GENERAL_SPLITTING,
    HORN_REVEALING,
   
    SATURATION,
   
    FINALIZATION,
    UNKNOWN_PHASE
  };

  ExecutionPhase phase;

private:
  static const char* phaseToString(ExecutionPhase p);
}; 

}

#endif
