
/*
 * File TPLexer.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __TPLexer__
#define __TPLexer__

#include <iostream>

#include "Lib/Array.hpp"
#include "Lib/Stack.hpp"
#include "Lib/Exception.hpp"

#include "Token.hpp"

using namespace std;
using namespace Lib;

namespace Shell {

class TPLexer;

class TPLexer 
{
public:
 
 
  enum Tag {
   
    TEOF,
   
    NAME,
   
    VAR,
   
    LPAR,
   
    RPAR,
   
    LBRA,
   
    RBRA,
   
    COMMA,
   
    COLON,
   
    NOT,
   
    AND,
   
    EQUAL,
   
    STRING,
   
    NEQ,
   
    FORALL,
   
    EXISTS,
   
    PI,
   
    SIGMA,
   
    IMPLY,
   
    XOR,
   
    IFF,
   
    REVERSE_IMP,
   
    DOT,
   
    REAL,
   
    RAT,
   
    INT,
   
    OR,
   
    ASS,
   
    LAMBDA,
   
    APP,
   
    STAR,
   
    UNION,
   
    ARROW,
   
    SUBTYPE,
   
    NOT_OR,
   
    NOT_AND,
   
    SEQUENT,
   
    THF_QUANT_ALL,
   
    THF_QUANT_SOME,
   
    APP_PLUS,
   
    APP_MINUS,
  };

 
  struct Token {
   
    Tag tag;
   
    int start;
  };

  class Exception 
    : public ::Exception
  {
  public:
    Exception(TPLexer&,const char* message);
    void cry(ostream&);
    ~Exception() {}
  protected:
    string _message;
  }; 
  friend class Exception;

  TPLexer(istream& in);
  void parse();
 
  const char* input() { return _chars.content(); }

private:
 
  istream& _in;
 
  Array<char> _chars;
 
  Stack<int> _eols;
 
  Array<Token> _tokens;
 
  int _cpos;
 
  int _tpos;

  
  
  
  
  
  
  
  
  
  
  
  

  bool readToken();
  void skipWhiteSpacesAndComments();
  void readName();
  void readString();
  void readAtom();
  Tag readNumber();
  void decimal();
  void positiveDecimal();

   
  
  
  
  
  
  
  
  
  

  
  
  
  
  
  
  
  
  
  
  
  
  
}; 

}

#endif

