
/*
 * File TheoryAxioms.hpp.
 *
 * This file is part of the source code of the software program
 * Vampire as used in CASC-J6. It is protected by applicable
 * copyright laws.
 *
 * Posession of, or access to, this program does not
 * give you or anybody else a right or licence to
 * distribute, modify, copy, compile, create derivatives,
 * or use in any form or for any purpose, this program,
 * or any part thereof.
 * Any licence to use Vampire shall only be obtained
 * as described on Vampire's home page http://www.vprover.org.
 *
 * This file was included in the Vampire source code
 * for CASC-J6 only because so is required by the CASC-J6
 * rules, and for no other reason.
 */


#ifndef __TheoryAxioms__
#define __TheoryAxioms__

#include "Forwards.hpp"

#include "Kernel/Theory.hpp"

namespace Shell {

using namespace Lib;
using namespace Kernel;

class TheoryAxioms {
public:


  struct Arithmetic;

  void apply(Problem& prb);
  bool apply(UnitList*& units, Property* prop);

private:
  void addCommutativity(Interpretation op, UnitList*& units);
  void addAssociativity(Interpretation op, UnitList*& units);
  void addIdentity(Interpretation op, TermList idElement, UnitList*& units);
  void addCommutativeGroupAxioms(Interpretation op, Interpretation inverse, TermList idElement, UnitList*& units);

  void addReflexivity(Interpretation op, UnitList*& units);
  void addTransitivity(Interpretation op, UnitList*& units);
  void addOrderingTotality(Interpretation lessEqual, UnitList*& units);
  void addTotalOrderAxioms(Interpretation lessEqual, UnitList*& units);

  void addMonotonicity(Interpretation lessEqual, Interpretation addition, UnitList*& units);

  void addAdditionAndOrderingAxioms(Interpretation plus, Interpretation unaryMinus,
      TermList zeroElement, TermList oneElement, Interpretation lessEqual, UnitList*& units);
  void addAdditionOrderingAndMultiplicationAxioms(Interpretation plus, Interpretation unaryMinus,
      TermList zeroElement, TermList oneElement, Interpretation lessEqual, Interpretation multiply,
      UnitList*& units);

  void addExtraIntegerOrderingAxiom(Interpretation plus, TermList oneElement, Interpretation lessEqual,
      UnitList*& units);

  void addTheoryUnit(Literal* lit, UnitList*& units);
  void addTheoryClause(UnitList*& units, Literal* lit1, Literal* lit2, Literal* lit3=0);
};

}

#endif 
